@@ -26,7 +26,6 @@ namespace Mono.Debugger.Backend
 		protected readonly ProcessStart start;
 
 		protected readonly BfdContainer bfd_container;
-		protected readonly SymbolTableCollection symtab_collection;
 		protected readonly ProcessServant process;
 		protected readonly DebuggerErrorHandler error_handler;
 		protected readonly BreakpointManager breakpoint_manager;
@@ -66,7 +65,7 @@ namespace Mono.Debugger.Backend
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_io_thread_main (IntPtr io_data, ChildOutputHandler output_handler);
 		[DllImport("monodebuggerserver")]
-		static extern TargetError mono_debugger_server_spawn (IntPtr handle, string working_directory, string[] argv, string[] envp, out int child_pid, out IntPtr io_data, out IntPtr error);
+		static extern TargetError mono_debugger_server_spawn (IntPtr handle, string working_directory, string[] argv, string[] envp, bool redirect_fds, out int child_pid, out IntPtr io_data, out IntPtr error);
 
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_attach (IntPtr handle, int child_pid);
@@ -83,6 +82,9 @@ namespace Mono.Debugger.Backend
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_continue (IntPtr handle);
 
+		[DllImport("monodebuggerserver")]
+		static extern TargetError mono_debugger_server_resume (IntPtr handle);
+
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_detach (IntPtr handle);
 
@@ -114,7 +116,7 @@ namespace Mono.Debugger.Backend
 		static extern TargetError mono_debugger_server_mark_rti_frame (IntPtr handle);
 
 		[DllImport("monodebuggerserver")]
-		static extern TargetError mono_debugger_server_abort_invoke (IntPtr handle, long stack_pointer);
+		static extern TargetError mono_debugger_server_abort_invoke (IntPtr handle, long rti_id);
 
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_call_method_invoke (IntPtr handle, long invoke_method, long method_address, int num_params, int blob_size, IntPtr param_data, IntPtr offset_data, IntPtr blob_data, long callback_argument, bool debug);
@@ -152,6 +154,9 @@ namespace Mono.Debugger.Backend
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_set_signal (IntPtr handle, int signal, int send_it);
 
+		[DllImport("monodebuggerserver")]
+		static extern TargetError mono_debugger_server_get_pending_signal (IntPtr handle, out int signal);
+
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_kill (IntPtr handle);
 
@@ -180,7 +185,7 @@ namespace Mono.Debugger.Backend
 		static extern TargetError mono_debugger_server_pop_registers (IntPtr handle);
 
 		[DllImport("monodebuggerserver")]
-		static extern TargetError mono_debugger_server_get_callback_frame (IntPtr handle, long stack_pointer, bool exact_match, IntPtr registers);
+		static extern TargetError mono_debugger_server_get_callback_frame (IntPtr handle, long stack_pointer, bool exact_match, IntPtr info);
 
 		[DllImport("monodebuggerserver")]
 		static extern void mono_debugger_server_set_runtime_info (IntPtr handle, IntPtr mono_runtime_info);
@@ -201,6 +206,7 @@ namespace Mono.Debugger.Backend
 			CHILD_CALLED_EXIT,
 			CHILD_NOTIFICATION,
 			CHILD_INTERRUPTED,
+			RUNTIME_INVOKE_DONE,
 
 			UNHANDLED_EXCEPTION	= 4001,
 			THROW_EXCEPTION,
@@ -256,6 +262,13 @@ namespace Mono.Debugger.Backend
 			public int SIGINT;
 			public int SIGCHLD;
 
+			public int SIGFPE;
+			public int SIGQUIT;
+			public int SIGABRT;
+			public int SIGSEGV;
+			public int SIGILL;
+			public int SIGBUS;
+
 			public int Kernel_SIGRTMIN;
 			public int MonoThreadAbortSignal;
 
@@ -500,14 +513,10 @@ namespace Mono.Debugger.Backend
 			check_error (mono_debugger_server_mark_rti_frame (server_handle));
 		}
 
-		public bool AbortInvoke (TargetAddress stack_pointer)
+		public void AbortInvoke (long rti_id)
 		{
-			TargetError result = mono_debugger_server_abort_invoke (
-				server_handle, stack_pointer.Address);
-			if (result == TargetError.NoCallbackFrame)
-				return false;
+			TargetError result = mono_debugger_server_abort_invoke (server_handle, rti_id);
 			check_error (result);
-			return true;
 		}
 
 		public int InsertBreakpoint (TargetAddress address)
@@ -599,7 +608,7 @@ namespace Mono.Debugger.Backend
 
 			TargetError result = mono_debugger_server_spawn (
 				server_handle, start.WorkingDirectory, start.CommandLineArguments,
-				start.Environment, out child_pid, out io_data, out error);
+				start.Environment, start.RedirectOutput, out child_pid, out io_data, out error);
 			if (result != TargetError.None) {
 				string message = Marshal.PtrToStringAuto (error);
 				g_free (error);
@@ -608,9 +617,11 @@ namespace Mono.Debugger.Backend
 					TargetError.CannotStartTarget, message);
 			}
 
-			ST.Thread io_thread = new ST.Thread (new ST.ThreadStart (io_thread_main));
-			io_thread.IsBackground = true;
-			io_thread.Start ();
+			if (start.RedirectOutput) {
+				ST.Thread io_thread = new ST.Thread (new ST.ThreadStart (io_thread_main));
+				io_thread.IsBackground = true;
+				io_thread.Start ();
+			}
 
 			return child_pid;
 		}
@@ -696,6 +707,7 @@ namespace Mono.Debugger.Backend
 
 			case ChildEventType.CHILD_CALLBACK:
 			case ChildEventType.CHILD_CALLBACK_COMPLETED:
+			case ChildEventType.RUNTIME_INVOKE_DONE:
 			case ChildEventType.CHILD_STOPPED:
 			case ChildEventType.CHILD_INTERRUPTED:
 			case ChildEventType.CHILD_HIT_BREAKPOINT:
@@ -1127,6 +1139,19 @@ namespace Mono.Debugger.Backend
 			}
 		}
 
+		public void Resume ()
+		{
+			check_disposed ();
+
+			TargetState old_state = change_target_state (TargetState.Running);
+			try {
+				check_error (mono_debugger_server_resume (server_handle));
+			} catch {
+				change_target_state (old_state);
+				throw;
+			}
+		}
+
 		// <summary>
 		//   Stop the inferior.
 		//   Returns true if it actually stopped the inferior and false if it was
@@ -1170,6 +1195,14 @@ namespace Mono.Debugger.Backend
 			check_error (mono_debugger_server_set_signal (server_handle, signal, do_send));
 		}
 
+		public int GetPendingSignal ()
+		{
+			int signal;
+			check_disposed ();
+			check_error (mono_debugger_server_get_pending_signal (server_handle, out signal));
+			return signal;
+		}
+
 		public void Detach ()
 		{
 			check_disposed ();
@@ -1327,28 +1360,59 @@ namespace Mono.Debugger.Backend
 			check_error (mono_debugger_server_pop_registers (server_handle));
 		}
 
-		internal Registers GetCallbackFrame (TargetAddress stack_pointer, bool exact_match)
+		internal CallbackFrame GetCallbackFrame (TargetAddress stack_pointer, bool exact_match)
 		{
 			IntPtr buffer = IntPtr.Zero;
 			try {
 				int count = arch.CountRegisters;
-				int buffer_size = count * 8;
+				int buffer_size = 32 + count * 8;
 				buffer = Marshal.AllocHGlobal (buffer_size);
 				TargetError result = mono_debugger_server_get_callback_frame (
 					server_handle, stack_pointer.Address, exact_match, buffer);
 				if (result == TargetError.NoCallbackFrame)
 					return null;
 				check_error (result);
-				long[] retval = new long [count];
-				Marshal.Copy (buffer, retval, 0, count);
 
-				return new Registers (arch, retval);
+				return new CallbackFrame (this, buffer);
 			} finally {
 				if (buffer != IntPtr.Zero)
 					Marshal.FreeHGlobal (buffer);
 			}
 		}
 
+		internal class CallbackFrame
+		{
+			public readonly long ID;
+			public readonly TargetAddress CallAddress;
+			public readonly TargetAddress StackPointer;
+			public readonly bool IsRuntimeInvokeFrame;
+			public readonly bool IsExactMatch;
+			public readonly Registers Registers;
+
+			public CallbackFrame (Inferior inferior, IntPtr data)
+			{
+				ID = Marshal.ReadInt64 (data);
+				CallAddress = new TargetAddress (inferior.AddressDomain, Marshal.ReadInt64 (data, 8));
+				StackPointer = new TargetAddress (inferior.AddressDomain, Marshal.ReadInt64 (data, 16));
+
+				int flags = Marshal.ReadInt32 (data, 24);
+				IsRuntimeInvokeFrame = (flags & 1) == 1;
+				IsExactMatch = (flags & 2) == 2;
+
+				long[] regs = new long [inferior.arch.CountRegisters];
+				for (int i = 0; i < regs.Length; i++)
+					regs [i] = Marshal.ReadInt64 (data, 32 + 8 * i);
+
+				Registers = new Registers (inferior.arch, regs);
+			}
+
+			public override string ToString ()
+			{
+				return String.Format ("Inferior.CallbackFrame ({0}:{1:x}:{2:x}:{3})", ID,
+						      CallAddress, StackPointer, IsRuntimeInvokeFrame);
+			}
+		}
+
 		internal void SetRuntimeInfo (IntPtr mono_runtime_info)
 		{
 			mono_debugger_server_set_runtime_info (server_handle, mono_runtime_info);
@@ -1536,6 +1600,19 @@ namespace Mono.Debugger.Backend
 			}
 		}
 
+		public bool IsManagedSignal (int signal)
+		{
+			if (!has_signals)
+				throw new InvalidOperationException ();
+
+			if ((signal == signal_info.SIGFPE) || (signal == signal_info.SIGQUIT) ||
+			    (signal == signal_info.SIGABRT) || (signal == signal_info.SIGSEGV) ||
+			    (signal == signal_info.SIGILL) || (signal == signal_info.SIGBUS))
+				return true;
+
+			return false;
+		}
+
 		/*
 		 * CAUTION: This is the hard limit of the Linux kernel, not the first
 		 *          user-visible real-time signal !
