@@ -182,13 +182,13 @@ namespace Mono.Debugger.Backend
 			SingleSteppingEngine new_thread = new SingleSteppingEngine (
 				manager, this, new_inferior, pid);
 
-			Report.Debug (DebugFlags.Threads, "Thread created: {0} {1}", pid, new_thread);
+			Report.Debug (DebugFlags.Threads, "Thread created: {0} {1} {2}", pid, new_thread, do_attach);
 
 			// Order is important: first add the new engine to the manager's hash table,
 			//                     then call inferior.Initialize() / inferior.Attach().
 			manager.AddEngine (new_thread);
 
-			new_thread.StartThread (do_attach);
+			new_thread.StartThread (do_attach, true);
 
 			if ((mono_manager != null) && !do_attach)
 				mono_manager.ThreadCreated (new_thread);
@@ -274,7 +274,7 @@ namespace Mono.Debugger.Backend
 			thread_hash [inferior.PID] = new_thread;
 
 			manager.ProcessExecd (new_thread);
-			new_thread.StartThread (false);
+			new_thread.StartThread (false, false);
 
 			manager.Debugger.OnProcessExecdEvent (this);
 			manager.Debugger.OnThreadCreatedEvent (new_thread.Thread);
@@ -290,6 +290,7 @@ namespace Mono.Debugger.Backend
 		{
 			if (!is_forked)
 				session.OnProcessExited (client);
+			client.OnProcessExited ();
 			session.MainThreadGroup.RemoveThread (main_thread.ID);
 			manager.Debugger.OnProcessExitedEvent (this);
 		}
@@ -663,6 +664,14 @@ namespace Mono.Debugger.Backend
 				      "Released global thread lock #1: {0}", caller);
 		}
 
+		public void ActivatePendingBreakpoints (CommandResult result)
+		{
+			((SingleSteppingEngine) main_thread).ManagedCallback (
+				delegate (SingleSteppingEngine sse) {
+					return sse.ActivatePendingBreakpoints (null);
+				}, result);
+		}
+
 		//
 		// Stopping / resuming all threads for the GUI
 		//
@@ -672,6 +681,16 @@ namespace Mono.Debugger.Backend
 			client.OnTargetEvent (sse, args);
 		}
 
+		internal void OnEnterNestedBreakState (SingleSteppingEngine sse)
+		{
+			client.OnEnterNestedBreakState (sse);
+		}
+
+		internal void OnLeaveNestedBreakState (SingleSteppingEngine sse)
+		{
+			client.OnLeaveNestedBreakState (sse);
+		}
+
 		//
 		// IDisposable
 		//
@@ -708,11 +727,20 @@ namespace Mono.Debugger.Backend
 				}
 			}
 
-			if (breakpoint_manager != null)
+			if (breakpoint_manager != null) {
 				breakpoint_manager.Dispose ();
+				breakpoint_manager = null;
+			}
 
-			if (thread_db != null)
+			if (thread_db != null) {
 				thread_db.Dispose ();
+				thread_db = null;
+			}
+
+			if (thread_lock_mutex != null) {
+				thread_lock_mutex.Dispose ();
+				thread_lock_mutex = null;
+			}
 
 			manager.RemoveProcess (this);
 		}
