@@ -1,11 +1,13 @@
 using System;
 using System.IO;
+using System.Linq;
 using System.Text;
 using ST = System.Threading;
 using System.Configuration;
 using System.Globalization;
 using System.Reflection;
 using System.Collections;
+using System.Collections.Generic;
 using System.Collections.Specialized;
 using System.Runtime.InteropServices;
 using System.Runtime.Remoting.Channels;
@@ -28,6 +30,9 @@ namespace Mono.Debugger.Backend
 			processes = ArrayList.Synchronized (new ArrayList ());
 
 			pending_events = Hashtable.Synchronized (new Hashtable ());
+
+			last_pending_sigstop = DateTime.Now;
+			pending_sigstops = new Dictionary<int,DateTime> ();
 			
 			address_domain = AddressDomain.Global;
 
@@ -68,6 +73,9 @@ namespace Mono.Debugger.Backend
 
 		AddressDomain address_domain;
 
+		DateTime last_pending_sigstop;
+		Dictionary<int,DateTime> pending_sigstops;
+
 		DebuggerMutex command_mutex;
 		bool abort_requested;
 		bool waiting;
@@ -81,6 +89,9 @@ namespace Mono.Debugger.Backend
 		[DllImport("monodebuggerserver")]
 		static extern int mono_debugger_server_get_pending_sigint ();
 
+		[DllImport("monodebuggerserver")]
+		static extern Inferior.ChildEventType mono_debugger_server_dispatch_simple (int status, out int arg);
+
 		void start_inferior ()
 		{
 			event_queue.Lock ();
@@ -160,7 +171,10 @@ namespace Mono.Debugger.Backend
 			inferior.Process.Initialize (engine, inferior, false);
 
 			if (cevent.Type == Inferior.ChildEventType.CHILD_CREATED_THREAD) {
-				inferior.Process.ThreadCreated (inferior, (int) cevent.Argument, false);
+				int pid = (int) cevent.Argument;
+				if (pending_sigstops.ContainsKey (pid))
+					pending_sigstops.Remove (pid);
+				inferior.Process.ThreadCreated (inferior, pid, false);
 				resume_target = true;
 				return true;
 			}
@@ -434,6 +448,9 @@ namespace Mono.Debugger.Backend
 				Report.Debug (DebugFlags.Threads, "Wait thread abort: {0}",
 					      DebuggerWaitHandle.CurrentThread);
 				ST.Thread.ResetAbort ();
+			} catch (Exception ex) {
+				Report.Debug (DebugFlags.Threads, "FUCK: {0}", ex);
+				throw;
 			}
 
 			Report.Debug (DebugFlags.Threads, "Wait thread exiting: {0}",
@@ -446,7 +463,10 @@ namespace Mono.Debugger.Backend
 			wait_event.WaitOne ();
 			waiting = true;
 
-			int pid, status;
+		again:
+			Report.Debug (DebugFlags.Wait, "Wait thread again");
+
+			int pid = 0, status = 0;
 			if (abort_requested) {
 				Report.Debug (DebugFlags.Wait,
 					      "Wait thread abort requested");
@@ -465,6 +485,15 @@ namespace Mono.Debugger.Backend
 				return false;
 			}
 
+			if (DateTime.Now - last_pending_sigstop > new TimeSpan (0, 2, 30)) {
+				foreach (int pending in pending_sigstops.Keys) {
+					Report.Error ("Got SIGSTOP from unknown PID {0}!", pending);
+				}
+
+				pending_sigstops.Clear ();
+				last_pending_sigstop = DateTime.Now;
+			}
+
 			Report.Debug (DebugFlags.Wait, "Wait thread waiting");
 
 			//
@@ -487,11 +516,35 @@ namespace Mono.Debugger.Backend
 
 			SingleSteppingEngine event_engine = (SingleSteppingEngine) thread_hash [pid];
 			if (event_engine == null) {
-				Console.WriteLine ("WARNING: Got event {0:x} for unknown pid {1}",
-						   status, pid);
-				waiting = false;
-				RequestWait ();
-				return true;
+				int arg;
+				Inferior.ChildEventType etype = mono_debugger_server_dispatch_simple (status, out arg);
+
+				/*
+				 * There is a race condition in the Linux kernel which shows up on >= 2.6.27:
+				 *
+				 * When creating a new thread, the initial stopping event of that thread is sometimes
+				 * sent before sending the `PTRACE_EVENT_CLONE' for it.
+				 *
+				 * Because of this, we explicitly wait for the new thread to stop and ignore any
+				 * "early" stopping signals.
+				 *
+				 * See also the comments in _server_ptrace_wait_for_new_thread() in x86-linux-ptrace.c
+				 * and bugs #423518 and #466012.
+				 *
+				 */
+
+				if ((etype != Inferior.ChildEventType.CHILD_STOPPED) || (arg != 0)) {
+					Report.Error ("WARNING: Got event {0:x} for unknown pid {1}", status, pid);
+					waiting = false;
+					RequestWait ();
+					return true;
+				}
+
+				if (!pending_sigstops.ContainsKey (pid))
+					pending_sigstops.Add (pid, DateTime.Now);
+
+				Report.Debug (DebugFlags.Wait, "Ignoring SIGSTOP from unknown pid {0}.", pid);
+				goto again;
 			}
 
 			engine_event.WaitOne ();
@@ -500,7 +553,7 @@ namespace Mono.Debugger.Backend
 			engine_event.Reset ();
 
 			if (current_event != null) {
-				Console.WriteLine ("FUCK: {0}", Environment.StackTrace);
+				Console.WriteLine ("Current_event is not null: {0}", Environment.StackTrace);
 				throw new InternalError ();
 			}
 
@@ -518,9 +571,24 @@ namespace Mono.Debugger.Backend
 		{
 			if (waiting)
 				throw new InternalError ();
+			Report.Debug (DebugFlags.Wait, "Signalling wait event");
 			wait_event.Set ();
 		}
 
+		protected struct PendingEventInfo
+		{
+			public readonly int PID;
+			public readonly int Status;
+			public readonly DateTime TimeStamp;
+
+			public PendingEventInfo (int pid, int status)
+			{
+				this.PID = pid;
+				this.Status = status;
+				this.TimeStamp = DateTime.Now;
+			}
+		}
+
 #region IDisposable implementation
 		private bool disposed = false;
 
