@@ -353,8 +353,7 @@ namespace Mono.Debugger.Architectures
 				rbp -= addr_size;
 			}
 
-			return CreateFrame (frame.Thread, memory, new_rip, new_rsp, new_rbp,
-					    regs, true);
+			return CreateFrame (frame.Thread, FrameType.Normal, memory, new_rip, new_rsp, new_rbp, regs);
 		}
 
 		StackFrame read_prologue (StackFrame frame, TargetMemoryAccess memory,
@@ -386,8 +385,7 @@ namespace Mono.Debugger.Architectures
 
 				regs [(int) X86_Register.RSP].SetValue (new_rsp);
 
-				return CreateFrame (frame.Thread, memory, new_rip, new_rsp, new_rbp,
-						    regs, true);
+				return CreateFrame (frame.Thread, FrameType.Normal, memory, new_rip, new_rsp, new_rbp, regs);
 			}
 
 			// push %ebp
@@ -410,8 +408,7 @@ namespace Mono.Debugger.Architectures
 
 				regs [(int) X86_Register.RSP].SetValue (new_rsp);
 
-				return CreateFrame (frame.Thread, memory, new_rip, new_rsp,
-						    new_rbp, regs, true);
+				return CreateFrame (frame.Thread, FrameType.Normal, memory, new_rip, new_rsp, new_rbp, regs);
 			}
 
 			if (code [pos++] != 0x48) {
@@ -456,8 +453,7 @@ namespace Mono.Debugger.Architectures
 
 			rbp -= addr_size;
 
-			return CreateFrame (frame.Thread, memory, new_rip, new_rsp, new_rbp,
-					    regs, true);
+			return CreateFrame (frame.Thread, FrameType.Normal, memory, new_rip, new_rsp, new_rbp, regs);
 		}
 
 		StackFrame try_unwind_sigreturn (StackFrame frame, TargetMemoryAccess memory)
@@ -512,7 +508,7 @@ namespace Mono.Debugger.Architectures
 			Symbol name = new Symbol ("<signal handler>", rip, 0);
 
 			return new StackFrame (
-				frame.Thread, rip, rsp, rbp, regs, frame.Thread.NativeLanguage, name);
+				frame.Thread, FrameType.Signal, rip, rsp, rbp, regs, frame.Thread.NativeLanguage, name);
 		}
 
 		internal override StackFrame TrySpecialUnwind (StackFrame frame,
@@ -540,33 +536,7 @@ namespace Mono.Debugger.Architectures
 			inferior.SetRegisters (regs);
 		}
 
-		protected override TargetAddress AdjustReturnAddress (TargetMemoryAccess target,
-								      TargetAddress address)
-		{
-			TargetBinaryReader reader = target.ReadMemory (address-7, 7).GetReader ();
-
-			byte[] code = reader.ReadBuffer (7);
-			if (code [2] == 0xe8)
-				return address-5;
-
-			if ((code [1] == 0xff) &&
-			    ((code [2] & 0x38) == 0x10) && ((code [2] >> 6) == 2))
-				return ((code [0] & 0x40) == 0x40) ? address-7 : address-6;
-			else if ((code [4] == 0xff) &&
-				 ((code [5] & 0x38) == 0x10) && ((code [5] >> 6) == 1))
-				return ((code [3] & 0x40) == 0x40) ? address-4 : address-3;
-			else if ((code [5] == 0xff) &&
-				 ((code [6] & 0x38) == 0x10) && ((code [6] >> 6) == 3))
-				return ((code [4] & 0x40) == 0x40) ? address-3 : address-2;
-			else if ((code [5] == 0xff) &&
-				 ((code [6] & 0x38) == 0x10) && ((code [6] >> 6) == 0))
-				return ((code [4] & 0x40) == 0x40) ? address-3 : address-2;
-
-			return address;
-		}
-
-		internal override StackFrame CreateFrame (Thread thread, TargetMemoryAccess memory,
-							  Registers regs, bool adjust_retaddr)
+		internal override StackFrame CreateFrame (Thread thread, FrameType type, TargetMemoryAccess memory, Registers regs)
 		{
 			TargetAddress address = new TargetAddress (
 				memory.AddressDomain, regs [(int) X86_Register.RIP].GetValue ());
@@ -575,27 +545,20 @@ namespace Mono.Debugger.Architectures
 			TargetAddress frame_pointer = new TargetAddress (
 				memory.AddressDomain, regs [(int) X86_Register.RBP].GetValue ());
 
-			return CreateFrame (thread, memory, address, stack_pointer, frame_pointer,
-					    regs, adjust_retaddr);
+			return CreateFrame (thread, type, memory, address, stack_pointer, frame_pointer, regs);
 		}
 
-
-		internal override StackFrame GetLMF (ThreadServant thread, TargetMemoryAccess memory)
+		internal override StackFrame GetLMF (ThreadServant thread, TargetMemoryAccess memory, ref TargetAddress lmf_address)
 		{
-			TargetAddress lmf = memory.ReadAddress (thread.LMFAddress);
-			return GetLMF (thread, memory, lmf);
-		}
+			TargetAddress lmf = lmf_address;
+			TargetBinaryReader reader = memory.ReadMemory (lmf_address, 88).GetReader ();
 
-		StackFrame GetLMF (ThreadServant thread, TargetMemoryAccess memory, TargetAddress lmf)
-		{
-			TargetBinaryReader reader = memory.ReadMemory (lmf, 88).GetReader ();
-
-			reader.ReadTargetAddress (); // prev
-			TargetAddress lmf_addr = reader.ReadTargetAddress ();
+			lmf_address = reader.ReadTargetAddress (); // prev
+			reader.ReadTargetAddress ();
 			reader.ReadTargetAddress (); // method
 			TargetAddress rip = reader.ReadTargetAddress ();
 
-			if (lmf_addr.IsNull)
+			if (lmf_address.IsNull)
 				return null;
 
 			TargetAddress rbx = reader.ReadTargetAddress ();
@@ -607,19 +570,27 @@ namespace Mono.Debugger.Architectures
 			TargetAddress r15 = reader.ReadTargetAddress ();
 
 			Registers regs = new Registers (this);
-			regs [(int) X86_Register.RBX].SetValue (lmf + 24, rip);
+
+			if ((lmf_address.Address & 1) == 0) {
+				rip = memory.ReadAddress (rsp - 8);
+				regs [(int) X86_Register.RIP].SetValue (rsp - 8, rip);
+				regs [(int) X86_Register.RBP].SetValue (lmf + 40, rbp);
+			} else {
+				TargetAddress new_rbp = memory.ReadAddress (rbp);
+				regs [(int) X86_Register.RIP].SetValue (lmf + 24, rip);
+				regs [(int) X86_Register.RBP].SetValue (rbp, new_rbp);
+				rbp = new_rbp;
+				lmf_address--;
+			}
+				
 			regs [(int) X86_Register.RBX].SetValue (lmf + 32, rbx);
-			regs [(int) X86_Register.RBP].SetValue (lmf + 40, rbp);
 			regs [(int) X86_Register.RSP].SetValue (lmf + 48, rsp);
 			regs [(int) X86_Register.R12].SetValue (lmf + 56, r12);
 			regs [(int) X86_Register.R13].SetValue (lmf + 64, r13);
 			regs [(int) X86_Register.R14].SetValue (lmf + 72, r14);
 			regs [(int) X86_Register.R15].SetValue (lmf + 80, r15);
 
-			TargetAddress new_rbp = memory.ReadAddress (rbp);
-			regs [(int) X86_Register.RBP].SetValue (rbp, new_rbp);
-
-			return CreateFrame (thread.Client, memory, rip, rsp, new_rbp, regs, true);
+			return CreateFrame (thread.Client, FrameType.LMF, memory, rip, rsp, rbp, regs);
 		}
 	}
 }
