@@ -46,6 +46,7 @@ typedef struct
 	guint32 data_size;
 	int saved_signal;
 	gboolean debug;
+	gboolean is_rti;
 } CallbackData;
 
 ArchInfo *
@@ -422,6 +423,7 @@ server_ptrace_call_method_invoke (ServerHandle *handle, guint64 invoke_method,
 	cdata->exc_address = new_esp + 20;
 	cdata->callback_argument = callback_argument;
 	cdata->debug = debug;
+	cdata->is_rti = TRUE;
 	cdata->saved_signal = handle->inferior->last_signal;
 	handle->inferior->last_signal = 0;
 
@@ -507,6 +509,11 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 	if (stopsig == SIGSTOP)
 		return STOP_ACTION_INTERRUPTED;
 
+#if defined(__linux__) || defined(__FreeBSD__)
+	if (stopsig != SIGTRAP)
+		return STOP_ACTION_STOPPED;
+#endif
+
 	if (handle->mono_runtime &&
 	    ((guint32) INFERIOR_REG_EIP (arch->current_regs) - 1 == handle->mono_runtime->notification_address)) {
 		guint32 addr = (guint32) INFERIOR_REG_ESP (arch->current_regs) + 4;
@@ -594,6 +601,11 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 
 		x86_arch_get_registers (handle);
 
+		if (cdata->is_rti) {
+			g_free (cdata);
+			return STOP_ACTION_RTI_DONE;
+		}
+
 		if (cdata->debug) {
 			*retval = 0;
 			g_free (cdata);
@@ -628,11 +640,6 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 		return STOP_ACTION_STOPPED;
 	}
 
-#if defined(__linux__) || defined(__FreeBSD__)
-	if (stopsig != SIGTRAP)
-		return STOP_ACTION_STOPPED;
-#endif
-
 	if (server_ptrace_peek_word (handle, GPOINTER_TO_SIZE (INFERIOR_REG_EIP (arch->current_regs) - 1), &code) != COMMAND_ERROR_NONE)
 		return STOP_ACTION_STOPPED;
 
@@ -1221,17 +1228,12 @@ server_ptrace_mark_rti_frame (ServerHandle *handle)
 }
 
 static ServerCommandError
-server_ptrace_abort_invoke (ServerHandle *handle, guint64 stack_pointer)
+server_ptrace_abort_invoke (ServerHandle *handle, guint64 rti_id)
 {
 	CallbackData *cdata;
 
 	cdata = get_callback_data (handle->arch);
-	if (!cdata)
-		return COMMAND_ERROR_NO_CALLBACK_FRAME;
-
-	if (cdata->rti_frame && (stack_pointer < cdata->rti_frame))
-		return COMMAND_ERROR_NO_CALLBACK_FRAME;
-	if (stack_pointer < cdata->stack_pointer)
+	if (!cdata || !cdata->is_rti || (cdata->callback_argument != rti_id))
 		return COMMAND_ERROR_NO_CALLBACK_FRAME;
 
 	if (_server_ptrace_set_registers (handle->inferior, &cdata->saved_regs) != COMMAND_ERROR_NONE)
