@@ -7,10 +7,10 @@ G_BEGIN_DECLS
 
 #if defined(__i386__)
 
-#include <asm/user.h>
+#include <sys/user.h>
 
 #define INFERIOR_REGS_TYPE	struct user_regs_struct
-#define INFERIOR_FPREGS_TYPE	struct user_i387_struct
+#define INFERIOR_FPREGS_TYPE	struct user_fpregs_struct
 
 #define INFERIOR_REG_EIP(r)	r.eip
 #define INFERIOR_REG_ESP(r)	r.esp
@@ -21,14 +21,15 @@ G_BEGIN_DECLS
 #define INFERIOR_REG_EDX(r)	r.edx
 #define INFERIOR_REG_ESI(r)	r.esi
 #define INFERIOR_REG_EDI(r)	r.edi
+#define INFERIOR_REG_ORIG_EAX(r)	r.orig_eax
 #define INFERIOR_REG_EFLAGS(r)	r.eflags
 #define INFERIOR_REG_ESP(r)	r.esp
-#define INFERIOR_REG_FS(r)	r.fs
-#define INFERIOR_REG_ES(r)	r.es
-#define INFERIOR_REG_DS(r)	r.ds
-#define INFERIOR_REG_CS(r)	r.cs
-#define INFERIOR_REG_SS(r)	r.ss
-#define INFERIOR_REG_GS(r)	r.gs
+#define INFERIOR_REG_FS(r)	r.xfs
+#define INFERIOR_REG_ES(r)	r.xes
+#define INFERIOR_REG_DS(r)	r.xds
+#define INFERIOR_REG_CS(r)	r.xcs
+#define INFERIOR_REG_SS(r)	r.xss
+#define INFERIOR_REG_GS(r)	r.xgs
 
 G_END_DECLS
 
