@@ -7,6 +7,12 @@
 
 G_BEGIN_DECLS
 
+#ifdef __GNUC__
+#define MONO_ZERO_LEN_ARRAY 0
+#else
+#define MONO_ZERO_LEN_ARRAY 1
+#endif
+
 #define MONO_DEBUGGER_REMOTE_VERSION		1
 #define MONO_DEBUGGER_REMOTE_MAGIC		0x36885fe4
 
@@ -47,7 +53,8 @@ typedef enum {
 	MESSAGE_CHILD_EXECD,
 	MESSAGE_CHILD_CALLED_EXIT,
 	MESSAGE_CHILD_NOTIFICATION,
-	MESSAGE_CHILD_INTERRUPTED
+	MESSAGE_CHILD_INTERRUPTED,
+	MESSAGE_RUNTIME_INVOKE_DONE
 } ServerStatusMessageType;
 
 typedef struct {
@@ -61,6 +68,15 @@ typedef struct {
 	guint64 frame_address;
 } StackFrame;
 
+typedef struct {
+	guint64 callback_argument;
+	guint64 call_address;
+	guint64 stack_pointer;
+	guint64 is_rti_frame   : 1;
+	guint64 is_exact_match : 1;
+	guint64 saved_registers [MONO_ZERO_LEN_ARRAY];
+} CallbackInfo;
+
 #define EXECUTABLE_CODE_CHUNK_SIZE		16
 
 typedef struct
@@ -96,6 +112,12 @@ typedef struct {
 	int sigstop;
 	int sigint;
 	int sigchld;
+	int sigfpe;
+	int sigquit;
+	int sigabrt;
+	int sigsegv;
+	int sigill;
+	int sigbus;
 	int kernel_sigrtmin;
 	int mono_thread_abort;
 } SignalInfo;
@@ -138,6 +160,7 @@ struct InferiorVTable {
 						       const gchar        *working_directory,
 						       const gchar       **argv,
 						       const gchar       **envp,
+						       gboolean            redirect_fds,
 						       gint               *child_pid,
 						       IOThreadData      **io_data,
 						       gchar             **error);
@@ -162,6 +185,9 @@ struct InferiorVTable {
 						       guint32             *opt_data_size,
 						       gpointer            *opt_data);
 
+	ServerStatusMessageType (* dispatch_simple)   (guint32              status,
+						       guint32             *arg);
+
 	/* Get sizeof (int), sizeof (long) and sizeof (void *) from the target. */
 	ServerCommandError    (* get_target_info)     (guint32            *target_int_size,
 						       guint32            *target_long_size,
@@ -182,6 +208,8 @@ struct InferiorVTable {
 	 */
 	ServerCommandError    (* step)                (ServerHandle     *handle);
 
+	ServerCommandError    (* resume)              (ServerHandle     *handle);
+
 	/*
 	 * Get the current program counter.
 	 * Return COMMAND_ERROR_NOT_STOPPED if the target is currently running.
@@ -277,7 +305,7 @@ struct InferiorVTable {
 	ServerCommandError    (* mark_rti_frame)      (ServerHandle     *handle);
 
 	ServerCommandError    (* abort_invoke)        (ServerHandle     *handle,
-						       guint64           stack_pointer);
+						       guint64           rti_id);
 
 	/*
 	 * Insert a breakpoint at address `address' in the target's address space.
@@ -352,6 +380,9 @@ struct InferiorVTable {
 						       guint32           sig,
 						       guint32           send_it);
 
+	ServerCommandError    (* get_pending_signal)  (ServerHandle     *handle,
+						       guint32          *signal);
+
 	/*
 	 * Kill the target.
 	 */
@@ -380,7 +411,7 @@ struct InferiorVTable {
 	ServerCommandError    (* get_callback_frame)  (ServerHandle      *handle,
 						       guint64            stack_pointer,
 						       gboolean           exact_match,
-						       guint64           *registers);
+						       CallbackInfo      *info);
 
 	void         (* get_registers_from_core_file) (guint64           *values,
 						       const guint8      *buffer);
@@ -418,6 +449,7 @@ mono_debugger_server_spawn                (ServerHandle       *handle,
 					   const gchar        *working_directory,
 					   const gchar       **argv,
 					   const gchar       **envp,
+					   gboolean            redirect_fds,
 					   gint               *child_pid,
 					   IOThreadData      **io_data,
 					   gchar             **error);
@@ -461,6 +493,9 @@ mono_debugger_server_step                 (ServerHandle       *handle);
 ServerCommandError
 mono_debugger_server_continue             (ServerHandle       *handle);
 
+ServerCommandError
+mono_debugger_server_resume               (ServerHandle       *handle);
+
 ServerCommandError
 mono_debugger_server_detach               (ServerHandle       *handle);
 
@@ -536,7 +571,7 @@ mono_debugger_mark_rti_framenvoke        (ServerHandle        *handle);
 
 ServerCommandError
 mono_debugger_server_abort_invoke        (ServerHandle        *handle,
-					  guint64              stack_pointer);
+					  guint64              rti_id);
 
 ServerCommandError
 mono_debugger_server_insert_breakpoint   (ServerHandle        *handle,
@@ -582,6 +617,10 @@ mono_debugger_server_set_signal          (ServerHandle        *handle,
 					  guint32              sig,
 					  guint32              send_it);
 
+ServerCommandError
+mono_debugger_server_get_pending_signal  (ServerHandle        *handle,
+					  guint32             *signal);
+
 ServerCommandError
 mono_debugger_server_kill                (ServerHandle        *handle);
 
@@ -619,7 +658,7 @@ ServerCommandError
 mono_debugger_server_get_callback_frame  (ServerHandle        *handle,
 					  guint64              stack_pointer,
 					  gboolean             exact_match,
-					  guint64             *registers);
+					  CallbackInfo        *info);
 
 MonoRuntimeInfo *
 mono_debugger_server_initialize_mono_runtime (guint32 address_size,
