@@ -55,6 +55,7 @@ struct InferiorHandle
 #ifdef __linux__
 	int mem_fd;
 #endif
+	int stepping;
 	int last_signal;
 	int redirect_fds;
 	int output_fd [2], error_fd [2];
@@ -123,6 +124,7 @@ server_ptrace_continue (ServerHandle *handle)
 	InferiorHandle *inferior = handle->inferior;
 
 	errno = 0;
+	inferior->stepping = FALSE;
 	if (ptrace (PT_CONTINUE, inferior->pid, (caddr_t) 1, inferior->last_signal)) {
 		return _server_ptrace_check_errno (inferior);
 	}
@@ -136,12 +138,23 @@ server_ptrace_step (ServerHandle *handle)
 	InferiorHandle *inferior = handle->inferior;
 
 	errno = 0;
+	inferior->stepping = TRUE;
 	if (ptrace (PT_STEP, inferior->pid, (caddr_t) 1, inferior->last_signal))
 		return _server_ptrace_check_errno (inferior);
 
 	return COMMAND_ERROR_NONE;
 }
 
+static ServerCommandError
+server_ptrace_resume (ServerHandle *handle)
+{
+	InferiorHandle *inferior = handle->inferior;
+	if (inferior->stepping)
+		return server_ptrace_step (handle);
+	else
+		return server_ptrace_continue (handle);
+}
+
 static ServerCommandError
 server_ptrace_detach (ServerHandle *handle)
 {
@@ -158,6 +171,7 @@ server_ptrace_detach (ServerHandle *handle)
 static ServerCommandError
 server_ptrace_kill (ServerHandle *handle)
 {
+	kill (handle->inferior->pid, SIGKILL);
 	if (ptrace (PTRACE_KILL, handle->inferior->pid, NULL, 0))
 		return COMMAND_ERROR_UNKNOWN_ERROR;
 
@@ -327,6 +341,12 @@ server_ptrace_dispatch_event (ServerHandle *handle, guint32 status, guint64 *arg
 			*data1 = retval;
 			*data2 = retval2;
 			return MESSAGE_CHILD_NOTIFICATION;
+
+		case STOP_ACTION_RTI_DONE:
+			*arg = callback_arg;
+			*data1 = retval;
+			*data2 = retval2;
+			return MESSAGE_RUNTIME_INVOKE_DONE;
 		}
 
 		g_assert_not_reached ();
@@ -347,6 +367,37 @@ server_ptrace_dispatch_event (ServerHandle *handle, guint32 status, guint64 *arg
 	return MESSAGE_UNKNOWN_ERROR;
 }
 
+static ServerStatusMessageType
+server_ptrace_dispatch_simple (guint32 status, guint32 *arg)
+{
+	if (status >> 16)
+		return MESSAGE_UNKNOWN_ERROR;
+
+	if (WIFSTOPPED (status)) {
+		int stopsig = WSTOPSIG (status);
+
+		if ((stopsig == SIGSTOP) || (stopsig == SIGTRAP))
+			stopsig = 0;
+
+		*arg = stopsig;
+		return MESSAGE_CHILD_STOPPED;
+	} else if (WIFEXITED (status)) {
+		*arg = WEXITSTATUS (status);
+		return MESSAGE_CHILD_EXITED;
+	} else if (WIFSIGNALED (status)) {
+		if ((WTERMSIG (status) == SIGTRAP) || (WTERMSIG (status) == SIGKILL)) {
+			*arg = 0;
+			return MESSAGE_CHILD_EXITED;
+		} else {
+			*arg = WTERMSIG (status);
+			return MESSAGE_CHILD_SIGNALED;
+		}
+	}
+
+	return MESSAGE_UNKNOWN_ERROR;
+}
+
+
 static ServerHandle *
 server_ptrace_create_inferior (BreakpointManager *bpm)
 {
@@ -365,36 +416,42 @@ child_setup_func (InferiorHandle *inferior)
 	if (ptrace (PT_TRACE_ME, getpid (), NULL, 0))
 		g_error (G_STRLOC ": Can't PT_TRACEME: %s", g_strerror (errno));
 
-	dup2 (inferior->output_fd[1], 1);
-	dup2 (inferior->error_fd[1], 2);
+	if (inferior->redirect_fds) {
+		dup2 (inferior->output_fd[1], 1);
+		dup2 (inferior->error_fd[1], 2);
+	}
 }
 
 static ServerCommandError
 server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
-		     const gchar **argv, const gchar **envp, gint *child_pid,
-		     IOThreadData **io_data, gchar **error)
+		     const gchar **argv, const gchar **envp, gboolean redirect_fds,
+		     gint *child_pid, IOThreadData **io_data, gchar **error)
 {	
 	InferiorHandle *inferior = handle->inferior;
-	int fd[2], open_max, ret, len, i;
+	int fd[2], ret, len, i;
 	ServerCommandError result;
 
 	*error = NULL;
+	inferior->redirect_fds = redirect_fds;
+
+	if (redirect_fds) {
+		pipe (inferior->output_fd);
+		pipe (inferior->error_fd);
+
+		*io_data = g_new0 (IOThreadData, 1);
+		(*io_data)->output_fd = inferior->output_fd[0];
+		(*io_data)->error_fd = inferior->error_fd[0];
+	} else {
+		*io_data = NULL;
+	}
 
 	pipe (fd);
 
-	inferior->redirect_fds = TRUE;
-
-	pipe (inferior->output_fd);
-	pipe (inferior->error_fd);
-
-	*io_data = g_new0 (IOThreadData, 1);
-	(*io_data)->output_fd = inferior->output_fd[0];
-	(*io_data)->error_fd = inferior->error_fd[0];
-
 	*child_pid = fork ();
 	if (*child_pid == 0) {
 		gchar *error_message;
 		struct rlimit core_limit;
+		int open_max;
 
 		open_max = sysconf (_SC_OPEN_MAX);
 		for (i = 3; i < open_max; i++)
@@ -415,10 +472,12 @@ server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
 		write (fd [1], error_message, len);
 		_exit (1);
 	} else if (*child_pid < 0) {
-		close (inferior->output_fd[0]);
-		close (inferior->output_fd[1]);
-		close (inferior->error_fd[0]);
-		close (inferior->error_fd[1]);
+		if (redirect_fds) {
+			close (inferior->output_fd[0]);
+			close (inferior->output_fd[1]);
+			close (inferior->error_fd[0]);
+			close (inferior->error_fd[1]);
+		}
 		close (fd [0]);
 		close (fd [1]);
 
@@ -426,8 +485,10 @@ server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
 		return COMMAND_ERROR_CANNOT_START_TARGET;
 	}
 
-	close (inferior->output_fd[1]);
-	close (inferior->error_fd[1]);
+	if (redirect_fds) {
+		close (inferior->output_fd[1]);
+		close (inferior->error_fd[1]);
+	}
 	close (fd [1]);
 
 	ret = read (fd [0], &len, sizeof (len));
@@ -438,8 +499,10 @@ server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
 		*error = g_malloc0 (len);
 		read (fd [0], *error, len);
 		close (fd [0]);
-		close (inferior->output_fd[0]);
-		close (inferior->error_fd[0]);
+		if (redirect_fds) {
+			close (inferior->output_fd[0]);
+			close (inferior->error_fd[0]);
+		}
 		return COMMAND_ERROR_CANNOT_START_TARGET;
 	}
 
@@ -449,8 +512,10 @@ server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
 
 	result = _server_ptrace_setup_inferior (handle);
 	if (result != COMMAND_ERROR_NONE) {
-		close (inferior->output_fd[0]);
-		close (inferior->error_fd[0]);
+		if (redirect_fds) {
+			close (inferior->output_fd[0]);
+			close (inferior->error_fd[0]);
+		}
 		return result;
 	}
 
@@ -465,6 +530,9 @@ server_ptrace_initialize_thread (ServerHandle *handle, guint32 pid)
 	inferior->pid = pid;
 	inferior->is_thread = TRUE;
 
+	if (!_server_ptrace_wait_for_new_thread (handle))
+		return COMMAND_ERROR_INTERNAL_ERROR;
+
 	return _server_ptrace_setup_inferior (handle);
 }
 
@@ -543,6 +611,13 @@ server_ptrace_set_signal (ServerHandle *handle, guint32 sig, guint32 send_it)
 	return COMMAND_ERROR_NONE;
 }
 
+static ServerCommandError
+server_ptrace_get_pending_signal (ServerHandle *handle, guint32 *signal)
+{
+	*signal = handle->inferior->last_signal;
+	return COMMAND_ERROR_NONE;
+}
+
 static void
 server_ptrace_set_runtime_info (ServerHandle *handle, MonoRuntimeInfo *mono_runtime)
 {
@@ -582,9 +657,11 @@ InferiorVTable i386_ptrace_inferior = {
 	server_ptrace_global_wait,
 	server_ptrace_stop_and_wait,
 	server_ptrace_dispatch_event,
+	server_ptrace_dispatch_simple,
 	server_ptrace_get_target_info,
 	server_ptrace_continue,
 	server_ptrace_step,
+	server_ptrace_resume,
 	server_ptrace_get_frame,
 	server_ptrace_current_insn_is_bpt,
 	server_ptrace_peek_word,
@@ -608,6 +685,7 @@ InferiorVTable i386_ptrace_inferior = {
 	server_ptrace_set_registers,
 	server_ptrace_stop,
 	server_ptrace_set_signal,
+	server_ptrace_get_pending_signal,
 	server_ptrace_kill,
 	server_ptrace_get_signal_info,
 	server_ptrace_get_threads,
