@@ -49,6 +49,7 @@ typedef struct
 	guint64 data_pointer;
 	guint32 data_size;
 	gboolean debug;
+	gboolean is_rti;
 } CallbackData;
 
 ArchInfo *
@@ -186,33 +187,6 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 	if (stopsig == SIGSTOP)
 		return STOP_ACTION_INTERRUPTED;
 
-	if (handle->mono_runtime &&
-	    (INFERIOR_REG_RIP (arch->current_regs) - 1 == handle->mono_runtime->notification_address)) {
-		if (stopsig != SIGTRAP)
-			return STOP_ACTION_STOPPED;
-
-		*callback_arg = INFERIOR_REG_RDI (arch->current_regs);
-		*retval = INFERIOR_REG_RSI (arch->current_regs);
-		*retval2 = INFERIOR_REG_RDX (arch->current_regs);
-
-		return STOP_ACTION_NOTIFICATION;
-	}
-
-	for (i = 0; i < DR_NADDR; i++) {
-		if (X86_DR_WATCH_HIT (arch, i)) {
-			_server_ptrace_set_dr (inferior, DR_STATUS, 0);
-			arch->dr_status = 0;
-			*retval = arch->dr_regs [i];
-			return STOP_ACTION_BREAKPOINT_HIT;
-		}
-	}
-
-	if (check_breakpoint (handle, INFERIOR_REG_RIP (arch->current_regs) - 1, retval)) {
-		INFERIOR_REG_RIP (arch->current_regs)--;
-		_server_ptrace_set_registers (inferior, &arch->current_regs);
-		return STOP_ACTION_BREAKPOINT_HIT;
-	}
-
 	cdata = get_callback_data (arch);
 	if (cdata && (cdata->call_address == INFERIOR_REG_RIP (arch->current_regs))) {
 		if (cdata->pushed_registers) {
@@ -266,6 +240,11 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 
 		x86_arch_get_registers (handle);
 
+		if (cdata->is_rti) {
+			g_free (cdata);
+			return STOP_ACTION_RTI_DONE;
+		}
+
 		if (cdata->debug) {
 			*retval = 0;
 			g_free (cdata);
@@ -276,15 +255,55 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 		return STOP_ACTION_CALLBACK;
 	}
 
+#if defined(__linux__) || defined(__FreeBSD__)
+	if (stopsig != SIGTRAP)
+		return STOP_ACTION_STOPPED;
+#endif
+
+	if (handle->mono_runtime &&
+	    (INFERIOR_REG_RIP (arch->current_regs) - 1 == handle->mono_runtime->notification_address)) {
+		if (stopsig != SIGTRAP)
+			return STOP_ACTION_STOPPED;
+
+		*callback_arg = INFERIOR_REG_RDI (arch->current_regs);
+		*retval = INFERIOR_REG_RSI (arch->current_regs);
+		*retval2 = INFERIOR_REG_RDX (arch->current_regs);
+
+		return STOP_ACTION_NOTIFICATION;
+	}
+
+	for (i = 0; i < DR_NADDR; i++) {
+		if (X86_DR_WATCH_HIT (arch, i)) {
+			_server_ptrace_set_dr (inferior, DR_STATUS, 0);
+			arch->dr_status = 0;
+			*retval = arch->dr_regs [i];
+			return STOP_ACTION_BREAKPOINT_HIT;
+		}
+	}
+
+	if (check_breakpoint (handle, INFERIOR_REG_RIP (arch->current_regs) - 1, retval)) {
+		INFERIOR_REG_RIP (arch->current_regs)--;
+		_server_ptrace_set_registers (inferior, &arch->current_regs);
+		return STOP_ACTION_BREAKPOINT_HIT;
+	}
+
 	cbuffer = arch->code_buffer;
 	if (cbuffer) {
 		handle->mono_runtime->executable_code_bitfield [cbuffer->slot] = 0;
 
 		if (cbuffer->code_address + cbuffer->insn_size != INFERIOR_REG_RIP (arch->current_regs)) {
+			char buffer [1024];
+
 			g_warning (G_STRLOC ": %Lx,%d - %Lx - %Lx",
 				   cbuffer->code_address, cbuffer->insn_size,
 				   cbuffer->code_address + cbuffer->insn_size,
 				   INFERIOR_REG_RIP (arch->current_regs));
+
+			server_ptrace_read_memory (handle, cbuffer->code_address, 8, buffer);
+			g_warning (G_STRLOC ": %02hhx %02hhx %02hhx %02hhx %02hhx %02hhx %02hhx %02hhx",
+				   buffer [0], buffer [1], buffer [2], buffer [3], buffer [4],
+				   buffer [5], buffer [6], buffer [7]);
+
 			return STOP_ACTION_STOPPED;
 		}
 
@@ -300,11 +319,6 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 		return STOP_ACTION_STOPPED;
 	}
 
-#if defined(__linux__) || defined(__FreeBSD__)
-	if (stopsig != SIGTRAP)
-		return STOP_ACTION_STOPPED;
-#endif
-
 	if (server_ptrace_peek_word (handle, GPOINTER_TO_SIZE(INFERIOR_REG_RIP (arch->current_regs) - 1), &code) != COMMAND_ERROR_NONE)
 		return STOP_ACTION_STOPPED;
 
@@ -1157,6 +1171,7 @@ server_ptrace_call_method_invoke (ServerHandle *handle, guint64 invoke_method,
 	cdata->exc_address = new_rsp + 16;
 	cdata->callback_argument = callback_argument;
 	cdata->debug = debug;
+	cdata->is_rti = TRUE;
 	cdata->saved_signal = handle->inferior->last_signal;
 	handle->inferior->last_signal = 0;
 
@@ -1262,17 +1277,12 @@ server_ptrace_mark_rti_frame (ServerHandle *handle)
 }
 
 static ServerCommandError
-server_ptrace_abort_invoke (ServerHandle *handle, guint64 stack_pointer)
+server_ptrace_abort_invoke (ServerHandle *handle, guint64 rti_id)
 {
 	CallbackData *cdata;
 
 	cdata = get_callback_data (handle->arch);
-	if (!cdata)
-		return COMMAND_ERROR_NO_CALLBACK_FRAME;
-
-	if (cdata->rti_frame && (stack_pointer < cdata->rti_frame))
-		return COMMAND_ERROR_NO_CALLBACK_FRAME;
-	if (stack_pointer < cdata->stack_pointer)
+	if (!cdata || !cdata->is_rti || (cdata->callback_argument != rti_id))
 		return COMMAND_ERROR_NO_CALLBACK_FRAME;
 
 	if (_server_ptrace_set_registers (handle->inferior, &cdata->saved_regs) != COMMAND_ERROR_NONE)
@@ -1292,7 +1302,7 @@ server_ptrace_abort_invoke (ServerHandle *handle, guint64 stack_pointer)
 
 static ServerCommandError
 server_ptrace_get_callback_frame (ServerHandle *handle, guint64 stack_pointer,
-				  gboolean exact_match, guint64 *registers)
+				  gboolean exact_match, CallbackInfo *info)
 {
 	int i;
 
@@ -1307,6 +1317,9 @@ server_ptrace_get_callback_frame (ServerHandle *handle, guint64 stack_pointer,
 				if (cdata->rti_frame < stack_pointer)
 					continue;
 			}
+			info->is_rti_frame = 1;
+			info->is_exact_match = cdata->rti_frame == stack_pointer;
+			info->stack_pointer = cdata->rti_frame;
 		} else {
 			if (exact_match) {
 				if (cdata->stack_pointer != stack_pointer)
@@ -1315,35 +1328,41 @@ server_ptrace_get_callback_frame (ServerHandle *handle, guint64 stack_pointer,
 				if (cdata->stack_pointer < stack_pointer)
 					continue;
 			}
+			info->is_rti_frame = 0;
+			info->is_exact_match = cdata->stack_pointer == stack_pointer;
+			info->stack_pointer = cdata->stack_pointer;
 		}
 
-		registers [DEBUGGER_REG_R15] = (guint64) INFERIOR_REG_R15 (cdata->saved_regs);
-		registers [DEBUGGER_REG_R14] = (guint64) INFERIOR_REG_R14 (cdata->saved_regs);
-		registers [DEBUGGER_REG_R13] = (guint64) INFERIOR_REG_R13 (cdata->saved_regs);
-		registers [DEBUGGER_REG_R12] = (guint64) INFERIOR_REG_R12 (cdata->saved_regs);
-		registers [DEBUGGER_REG_RBP] = (guint64) INFERIOR_REG_RBP (cdata->saved_regs);
-		registers [DEBUGGER_REG_RBX] = (guint64) INFERIOR_REG_RBX (cdata->saved_regs);
-		registers [DEBUGGER_REG_R11] = (guint64) INFERIOR_REG_R11 (cdata->saved_regs);
-		registers [DEBUGGER_REG_R10] = (guint64) INFERIOR_REG_R10 (cdata->saved_regs);
-		registers [DEBUGGER_REG_R9] = (guint64) INFERIOR_REG_R9 (cdata->saved_regs);
-		registers [DEBUGGER_REG_R8] = (guint64) INFERIOR_REG_R8 (cdata->saved_regs);
-		registers [DEBUGGER_REG_RAX] = (guint64) INFERIOR_REG_RAX (cdata->saved_regs);
-		registers [DEBUGGER_REG_RCX] = (guint64) INFERIOR_REG_RCX (cdata->saved_regs);
-		registers [DEBUGGER_REG_RDX] = (guint64) INFERIOR_REG_RDX (cdata->saved_regs);
-		registers [DEBUGGER_REG_RSI] = (guint64) INFERIOR_REG_RSI (cdata->saved_regs);
-		registers [DEBUGGER_REG_RDI] = (guint64) INFERIOR_REG_RDI (cdata->saved_regs);
-		registers [DEBUGGER_REG_ORIG_RAX] = (guint64) INFERIOR_REG_ORIG_RAX (cdata->saved_regs);
-		registers [DEBUGGER_REG_RIP] = (guint64) INFERIOR_REG_RIP (cdata->saved_regs);
-		registers [DEBUGGER_REG_CS] = (guint64) INFERIOR_REG_CS (cdata->saved_regs);
-		registers [DEBUGGER_REG_EFLAGS] = (guint64) INFERIOR_REG_EFLAGS (cdata->saved_regs);
-		registers [DEBUGGER_REG_RSP] = (guint64) INFERIOR_REG_RSP (cdata->saved_regs);
-		registers [DEBUGGER_REG_SS] = (guint64) INFERIOR_REG_SS (cdata->saved_regs);
-		registers [DEBUGGER_REG_FS_BASE] = (guint64) INFERIOR_REG_FS_BASE (cdata->saved_regs);
-		registers [DEBUGGER_REG_GS_BASE] = (guint64) INFERIOR_REG_GS_BASE (cdata->saved_regs);
-		registers [DEBUGGER_REG_DS] = (guint64) INFERIOR_REG_DS (cdata->saved_regs);
-		registers [DEBUGGER_REG_ES] = (guint64) INFERIOR_REG_ES (cdata->saved_regs);
-		registers [DEBUGGER_REG_FS] = (guint64) INFERIOR_REG_FS (cdata->saved_regs);
-		registers [DEBUGGER_REG_GS] = (guint64) INFERIOR_REG_GS (cdata->saved_regs);
+		info->callback_argument = cdata->callback_argument;
+		info->call_address = cdata->call_address;
+
+		info->saved_registers [DEBUGGER_REG_R15] = (guint64) INFERIOR_REG_R15 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R14] = (guint64) INFERIOR_REG_R14 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R13] = (guint64) INFERIOR_REG_R13 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R12] = (guint64) INFERIOR_REG_R12 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RBP] = (guint64) INFERIOR_REG_RBP (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RBX] = (guint64) INFERIOR_REG_RBX (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R11] = (guint64) INFERIOR_REG_R11 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R10] = (guint64) INFERIOR_REG_R10 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R9] = (guint64) INFERIOR_REG_R9 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_R8] = (guint64) INFERIOR_REG_R8 (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RAX] = (guint64) INFERIOR_REG_RAX (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RCX] = (guint64) INFERIOR_REG_RCX (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RDX] = (guint64) INFERIOR_REG_RDX (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RSI] = (guint64) INFERIOR_REG_RSI (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RDI] = (guint64) INFERIOR_REG_RDI (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_ORIG_RAX] = (guint64) INFERIOR_REG_ORIG_RAX (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RIP] = (guint64) INFERIOR_REG_RIP (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_CS] = (guint64) INFERIOR_REG_CS (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_EFLAGS] = (guint64) INFERIOR_REG_EFLAGS (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_RSP] = (guint64) INFERIOR_REG_RSP (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_SS] = (guint64) INFERIOR_REG_SS (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_FS_BASE] = (guint64) INFERIOR_REG_FS_BASE (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_GS_BASE] = (guint64) INFERIOR_REG_GS_BASE (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_DS] = (guint64) INFERIOR_REG_DS (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_ES] = (guint64) INFERIOR_REG_ES (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_FS] = (guint64) INFERIOR_REG_FS (cdata->saved_regs);
+		info->saved_registers [DEBUGGER_REG_GS] = (guint64) INFERIOR_REG_GS (cdata->saved_regs);
 
 		return COMMAND_ERROR_NONE;
 	}
