@@ -3,6 +3,7 @@ using System.IO;
 using System.Reflection;
 using System.Configuration;
 using System.Collections;
+using System.Collections.Generic;
 using System.Collections.Specialized;
 using System.Runtime.Serialization;
 using System.Runtime.Serialization.Formatters.Binary;
@@ -10,7 +11,7 @@ using System.Xml;
 using System.Xml.XPath;
 
 using Mono.Debugger.Languages;
-using Mono.Debugger.Backends;
+using Mono.Debugger.Backend;
 
 namespace Mono.Debugger
 {
@@ -30,7 +31,8 @@ namespace Mono.Debugger
 
 	public interface IExpressionParser
 	{
-		SourceLocation Parse (Thread target, LocationType type, string name);
+		SourceLocation ParseLocation (Thread target, StackFrame frame,
+					      LocationType type, string name);
 	}
 
 	[Serializable]
@@ -46,6 +48,10 @@ namespace Mono.Debugger
 		protected readonly Hashtable thread_groups;
 		protected readonly ThreadGroup main_thread_group;
 
+		protected readonly Dictionary<string,string> directory_maps;
+		protected readonly List<string> user_module_paths;
+		protected readonly List<string> user_modules;
+
 		Process main_process;
 		IExpressionParser parser;
 		XmlDocument saved_session;
@@ -62,6 +68,9 @@ namespace Mono.Debugger
 			displays = Hashtable.Synchronized (new Hashtable ());
 			thread_groups = Hashtable.Synchronized (new Hashtable ());
 			main_thread_group = CreateThreadGroup ("main");
+			directory_maps = new Dictionary<string,string> ();
+			user_module_paths = new List<string> ();
+			user_modules = new List<string> ();
 		}
 
 		public DebuggerSession (DebuggerConfiguration config, DebuggerOptions options,
@@ -69,6 +78,9 @@ namespace Mono.Debugger
 			: this (config, name, parser)
 		{
 			this.Options = options;
+
+			if (Options.StopInMain)
+				AddEvent (new MainMethodBreakpoint (this));
 		}
 
 		internal DebuggerSession Clone (DebuggerOptions new_options, string new_name)
@@ -131,10 +143,12 @@ namespace Mono.Debugger
 					IExpressionParser parser)
 			: this (config, "main", parser)
 		{
-			XmlValidatingReader reader = new XmlValidatingReader (new XmlTextReader (stream));
+			XmlReaderSettings settings = new XmlReaderSettings ();
 			Assembly ass = Assembly.GetExecutingAssembly ();
 			using (Stream schema = ass.GetManifestResourceStream ("DebuggerConfiguration"))
-				reader.Schemas.Add (null, new XmlTextReader (schema));
+				settings.Schemas.Add (null, new XmlTextReader (schema));
+
+			XmlReader reader = XmlReader.Create (stream, settings);
 
 			saved_session = new XmlDocument ();
 			saved_session.Load (reader);
@@ -150,6 +164,13 @@ namespace Mono.Debugger
 			XPathNodeIterator options_iter = session_iter.Current.Select ("Options/*");
 			Options = new DebuggerOptions (options_iter);
 
+			XPathNodeIterator dir_map_iter = session_iter.Current.Select ("DirectoryMap/Map");
+			while (dir_map_iter.MoveNext ()) {
+				string from = dir_map_iter.Current.GetAttribute ("from", "");
+				string to = dir_map_iter.Current.GetAttribute ("to", "");
+				directory_maps.Add (from, to);
+			}
+
 			LoadSession (nav);
 		}
 
@@ -184,10 +205,17 @@ namespace Mono.Debugger
 			return handle;
 		}
 
+		[Obsolete]
 		public Event InsertExceptionCatchPoint (Thread target, ThreadGroup group,
 							TargetType exception)
 		{
-			Event handle = new ExceptionCatchPoint (group, exception);
+			return InsertExceptionCatchPoint (target, group, exception, false);
+		}
+
+		public Event InsertExceptionCatchPoint (Thread target, ThreadGroup group,
+							TargetType exception, bool unhandled)
+		{
+			Event handle = new ExceptionCatchPoint (group, exception, unhandled);
 			handle.Activate (target);
 			AddEvent (handle);
 			return handle;
@@ -202,37 +230,52 @@ namespace Mono.Debugger
 			return handle;
 		}
 
-		internal SourceLocation ParseLocation (Thread target, LocationType type, string name)
+		internal SourceLocation ParseLocation (Thread target, StackFrame frame,
+						       LocationType type, string name)
 		{
-			return parser.Parse (target, type, name);
+			return parser.ParseLocation (target, frame, type, name);
+		}
+
+		public SourceFile FindFile (string filename)
+		{
+			if (main_process == null)
+				return null;
+
+			Module[] modules = main_process.Modules;
+
+			foreach (Module module in modules) {
+				SourceFile file = module.FindFile (filename);
+				if (file != null)
+					return file;
+			}
+
+			if (Config.OpaqueFileNames || Path.IsPathRooted (filename))
+				return null;
+
+			filename = Path.GetFullPath (Path.Combine (
+				Options.WorkingDirectory, filename));
+
+			foreach (Module module in modules) {
+				SourceFile file = module.FindFile (filename);
+				if (file != null)
+					return file;
+			}
+
+			return null;
 		}
 
 		//
 		// Session management.
 		//
 
-		//
-		// FIXME: Ideally, this would be called automatically from the
-		//        SingleSteppingEngine.  The problem is that `Event.Activate()' may
-		//        need to run stepping operations, so doing this correctly would
-		//        require some more work.  Keeping this as a quick fix for the
-		//        moment.
-		//
-		public void MainProcessReachedMain (Process process)
+		internal void OnMainProcessCreated (Process process)
 		{
-			main_process = process;
-
-			foreach (Event e in events.Values) {
-				if (!e.IsEnabled)
-					continue;
-
-				try {
-					e.Activate (process.MainThread);
-				} catch (TargetException ex) {
-					Report.Error ("Cannot insert breakpoint {0}: {1}",
-						      e.Index, ex.Message);
-				}
+			if (!process.IsManaged) {
+				Config.GetModuleGroup ("dll").StepInto = true;
+				Config.GetModuleGroup ("native").StepInto = true;
 			}
+
+			main_process = process;
 		}
 
 		internal void OnProcessCreated (Process process)
@@ -343,7 +386,10 @@ namespace Mono.Debugger
 				return new ExpressionBreakpoint (this, index, group, type, expression);
 			} else if (navigator.Name == "Exception") {
 				string exc = navigator.GetAttribute ("type", "");
-				return new ExceptionCatchPoint (index, group, exc);
+				bool unhandled = Boolean.Parse (navigator.GetAttribute ("unhandled", ""));
+				return new ExceptionCatchPoint (index, group, exc, unhandled);
+			} else if (navigator.Name == "MainMethod") {
+				return new MainMethodBreakpoint (this);
 			} else
 				throw new InternalError ();
 		}
@@ -369,6 +415,21 @@ namespace Mono.Debugger
 			return module;
 		}
 
+		bool is_user_module (string code_base)
+		{
+			foreach (string path in user_module_paths) {
+				if (code_base.StartsWith (path))
+					return true;
+			}
+
+			foreach (string user in user_modules) {
+				if (user == code_base)
+					return true;
+			}
+
+			return false;
+		}
+
 		internal Module CreateModule (string name, SymbolFile symfile)
 		{
 			if (symfile == null)
@@ -378,7 +439,11 @@ namespace Mono.Debugger
 			if (module != null)
 				return module;
 
-			ModuleGroup group = Config.GetModuleGroup (symfile);
+			ModuleGroup group;
+			if (is_user_module (symfile.CodeBase))
+				group = Config.GetModuleGroup ("user");
+			else
+				group = Config.GetModuleGroup (symfile);
 
 			module = new Module (group, name, symfile);
 			modules.Add (name, module);
@@ -457,7 +522,7 @@ namespace Mono.Debugger
 			return (Event) events [index];
 		}
 
-		internal void AddEvent (Event handle)
+		public void AddEvent (Event handle)
 		{
 			events.Add (handle.Index, handle);
 		}
@@ -497,5 +562,48 @@ namespace Mono.Debugger
 		{
 			displays.Remove (d.Index);
 		}
+
+		//
+		// File names and Directories
+		//
+
+		bool map_file_name (ref string path, string from, string to)
+		{
+			if (!path.StartsWith (from))
+				return false;
+
+			path = Path.Combine (to, path.Substring (from.Length + 1));
+			return true;
+		}
+
+		public string MapFileName (string path)
+		{
+			path = DebuggerConfiguration.WindowsToUnix (path);
+			foreach (KeyValuePair<string,string> map in directory_maps) {
+				if (map_file_name (ref path, map.Key, map.Value))
+					return path;
+			}
+
+			foreach (KeyValuePair<string,string> map in Config.DirectoryMaps) {
+				if (map_file_name (ref path, map.Key, map.Value))
+					return path;
+			}
+
+			return path;
+		}
+
+		//
+		// User modules
+		//
+
+		public void AddUserModule (string name)
+		{
+			user_modules.Add (name);
+		}
+
+		public void AddUserModulePath (string path)
+		{
+			user_module_paths.Add (path);
+		}
 	}
 }
