@@ -8,15 +8,12 @@ using System.Xml;
 using System.Xml.XPath;
 
 using Mono.Debugger.Languages;
-using Mono.Debugger.Backends;
+using Mono.Debugger.Backend;
 
 namespace Mono.Debugger
 {
 	public delegate void ModuleEventHandler (Module module);
 
-	internal delegate void MethodLoadedHandler (TargetMemoryAccess target, SourceMethod method,
-						    object user_data);
-
 	internal interface ILoadHandler
 	{
 		object UserData {
@@ -40,11 +37,11 @@ namespace Mono.Debugger
 			get;
 		}
 
-		public abstract Language Language {
+		public abstract string CodeBase {
 			get;
 		}
 
-		internal abstract ILanguageBackend LanguageBackend {
+		public abstract Language Language {
 			get;
 		}
 
@@ -60,13 +57,11 @@ namespace Mono.Debugger
 			get;
 		}
 
-		public abstract SourceMethod[] GetMethods (SourceFile file);
+		public abstract MethodSource[] GetMethods (SourceFile file);
 
-		public abstract Method GetMethod (int domain, long handle);
+		// public abstract Method GetMethod (int domain, long handle);
 
-		public abstract TargetFunctionType LookupMethod (string class_name, string name);
-
-		public abstract SourceMethod FindMethod (string name);
+		public abstract MethodSource FindMethod (string name);
 
 		public abstract Symbol SimpleLookup (TargetAddress address, bool exact_match);
 
@@ -74,11 +69,6 @@ namespace Mono.Debugger
 			get;
 		}
 
-		internal abstract ILoadHandler RegisterLoadHandler (Thread target,
-								    SourceMethod method,
-								    MethodLoadedHandler handler,
-								    object user_data);
-
 		internal abstract StackFrame UnwindStack (StackFrame last_frame,
 							  TargetMemoryAccess memory);
 
@@ -293,10 +283,6 @@ namespace Mono.Debugger
 			get { return SymbolFile.Language; }
 		}
 
-		internal ILanguageBackend LanguageBackend {
-			get { return SymbolFile.LanguageBackend; }
-		}
-
 		internal SymbolFile SymbolFile {
 			get {
 				if (symfile != null)
@@ -413,7 +399,7 @@ namespace Mono.Debugger
 			get { return SymbolFile.HasDebuggingInfo; }
 		}
 
-		protected void OnModuleChanged ()
+		void OnModuleChanged ()
 		{
 			if (symfile != null)
 				symfile.OnModuleChanged ();
@@ -423,26 +409,23 @@ namespace Mono.Debugger
 			get { return SymbolFile.Sources; }
 		}
 
-		public SourceMethod[] GetMethods (SourceFile file)
+		public MethodSource[] GetMethods (SourceFile file)
 		{
 			return SymbolFile.GetMethods (file);
 		}
 
+#if FIXME
 		public Method GetMethod (int domain, long handle)
 		{
 			return SymbolFile.GetMethod (domain, handle);
 		}
-
-		public TargetFunctionType LookupMethod (string class_name, string name)
-		{
-			return SymbolFile.LookupMethod (class_name, name);
-		}
+#endif
 
 		// <summary>
 		//   Find method @name, which must be a full method name including the
 		//   signature (System.DateTime.GetUtcOffset(System.DateTime)).
 		// </summary>
-		public SourceMethod FindMethod (string name)
+		public MethodSource FindMethod (string name)
 		{
 			if (!SymbolsLoaded)
 				return null;
