@@ -3,7 +3,7 @@ using System.Threading;
 using System.Collections;
 using System.Runtime.InteropServices;
 
-using Mono.Debugger.Backends;
+using Mono.Debugger.Backend;
 
 namespace Mono.Debugger
 {
