@@ -12,6 +12,16 @@ using Mono.Debugger.Languages.Mono;
 
 namespace Mono.Debugger
 {
+	public delegate bool ExceptionCatchPointHandler (string exception, out ExceptionAction action);
+
+	[Serializable]
+	public enum ExceptionAction
+	{
+		None = 0,
+		Stop = 1,
+		StopUnhandled = 2
+	}
+
 	public class Process : DebuggerMarshalByRefObject
 	{
 		Debugger debugger;
@@ -85,6 +95,11 @@ namespace Mono.Debugger
 			get { return servant.Modules; }
 		}
 
+		public Method Lookup (TargetAddress address)
+		{
+			return servant.SymbolTableManager.Lookup (address);
+		}
+
 		public TargetAddress LookupSymbol (string name)
 		{
 			return servant.LookupSymbol (name);
@@ -108,6 +123,46 @@ namespace Mono.Debugger
 				TargetOutputEvent (is_stderr, output);
 		}
 
+		//
+		// Test
+		//
+
+		public void ActivatePendingBreakpoints ()
+		{
+			ProcessCommandResult result = new ProcessCommandResult (this);
+			servant.ActivatePendingBreakpoints (result);
+			result.Wait ();
+		}
+
+		public class ProcessCommandResult : CommandResult
+		{
+			Process process;
+			ST.ManualResetEvent completed_event = new ST.ManualResetEvent (false);
+
+			internal ProcessCommandResult (Process process)
+			{
+				this.process = process;
+			}
+
+			public Process Process {
+				get { return process; }
+			}
+
+			public override ST.WaitHandle CompletedEvent {
+				get { return completed_event; }
+			}
+
+			internal override void Completed ()
+			{
+				completed_event.Set ();
+			}
+
+			public override void Abort ()
+			{
+				throw new NotImplementedException ();
+			}
+		}
+
 		//
 		// Stopping / resuming all threads for the GUI
 		//
@@ -132,6 +187,38 @@ namespace Mono.Debugger
 			Debugger.OnTargetEvent (sse.Client, args);
 		}
 
+		internal void OnEnterNestedBreakState (SingleSteppingEngine sse)
+		{
+			Debugger.OnEnterNestedBreakState (sse.Client);
+		}
+
+		internal void OnLeaveNestedBreakState (SingleSteppingEngine sse)
+		{
+			Debugger.OnLeaveNestedBreakState (sse.Client);
+		}
+
+		internal void OnProcessExited ()
+		{
+			if (manager != null)
+				manager.OnProcessExited (this);
+		}
+
+		ExceptionCatchPointHandler generic_exc_handler;
+
+		public void InstallGenericExceptionCatchPoint (ExceptionCatchPointHandler handler)
+		{
+			this.generic_exc_handler = handler;
+		}
+
+		public bool GenericExceptionCatchPoint (string exception, out ExceptionAction action)
+		{
+			if (generic_exc_handler != null)
+				return generic_exc_handler (exception, out action);
+
+			action = ExceptionAction.None;
+			return false;
+		}
+
 		//
 		// IDisposable
 		//
