@@ -6,14 +6,14 @@ using Mono.Debugger;
 
 namespace Mono.Debugger
 {
-	public class SourceFileFactory : MarshalByRefObject
+	public class SourceFileFactory : DebuggerMarshalByRefObject
 	{
 		Hashtable files = new Hashtable ();
 
-		public ISourceBuffer FindFile (string name)
+		public SourceBuffer FindFile (string name)
 		{
 			if (files.Contains (name))
-				return (ISourceBuffer) ((ObjectCache) files [name]).Data;
+				return (SourceBuffer) ((ObjectCache) files [name]).Data;
 
 			ObjectCache cache = (ObjectCache) files [name];
 			if (cache == null) {
@@ -21,7 +21,7 @@ namespace Mono.Debugger
 				files.Add (name, cache);
 			}
 
-			return (ISourceBuffer) cache.Data;
+			return (SourceBuffer) cache.Data;
 		}
 
 		public bool Exists (string name)
