@@ -4,9 +4,11 @@ using System.IO;
 using System.Reflection;
 using System.Collections;
 using System.Globalization;
+using ST=System.Threading;
 using Mono.Debugger;
 using Mono.Debugger.Languages;
 using Mono.Debugger.Languages.Mono;
+using EE=Mono.Debugger.ExpressionEvaluator;
 
 namespace Mono.Debugger.Frontend
 {
@@ -23,9 +25,21 @@ namespace Mono.Debugger.Frontend
 			parser = new CSharp.ExpressionParser ("C#");
 		}
 
-		public Expression Parse (string expression)
+		/*
+		 * This version throws an `ExpressionParsingException' containing a detailed
+		 * error message and the location of the error.
+		 *
+		 * Use ScriptingContext.ParseExpression() to get a `ScriptingException' with
+		 * a user-readable error message.
+		 */
+		internal Expression ParseInternal (string text)
 		{
-			return parser.Parse (expression);
+			return parser.Parse (text);
+		}
+
+		public EE.IExpression Parse (string text)
+		{
+			return new MyExpression (this, parser.Parse (text));
 		}
 
 		protected static SourceLocation FindFile (ScriptingContext context, string filename,
@@ -48,10 +62,7 @@ namespace Mono.Debugger.Frontend
 		protected SourceLocation DoParseExpression (ScriptingContext context,
 							    LocationType type, string arg)
 		{
-			Expression expr = Interpreter.ExpressionParser.Parse (arg);
-			if (expr == null)
-				throw new ScriptingException ("Cannot resolve expression `{0}'.", arg);
-
+			Expression expr = context.ParseExpression (arg);
 			MethodExpression mexpr = expr.ResolveMethod (context, type);
 
 			if (mexpr != null)
@@ -107,8 +118,8 @@ namespace Mono.Debugger.Frontend
 			return DoParseExpression (context, type, arg);
 		}
 
-		public SourceLocation Parse (Thread target, StackFrame frame,
-					     LocationType type, string arg)
+		public SourceLocation ParseLocation (Thread target, StackFrame frame,
+						     LocationType type, string arg)
 		{
 			ScriptingContext context = new ScriptingContext (Interpreter);
 			context.CurrentThread = target;
@@ -124,16 +135,7 @@ namespace Mono.Debugger.Frontend
 		public string EvaluateExpression (ScriptingContext context, string text,
 						  DisplayFormat format)
 		{
-			Expression expression;
-
-			try {
-				expression = Interpreter.ExpressionParser.Parse (text);
-			} catch (ScriptingException ex) {
-				throw new ScriptingException ("Cannot parse expression `{0}': {1}.",
-							      text, ex.Message);
-			} catch {
-				throw new ScriptingException ("Cannot parse expression `{0}'.", text);
-			}
+			Expression expression = context.ParseExpression (text);
 
 			try {
 				expression = expression.Resolve (context);
@@ -154,6 +156,127 @@ namespace Mono.Debugger.Frontend
 				throw new ScriptingException ("Cannot evaluate expression `{0}'.", text);
 			}
 		}
+
+		protected MyAsyncResult EvaluateAsync (StackFrame frame, MyExpression expression,
+						       EE.EvaluationFlags flags, EE.EvaluationCallback callback)
+		{
+			MyAsyncResult async = new MyAsyncResult (expression);
+
+			ST.ThreadPool.QueueUserWorkItem (delegate {
+				ScriptingContext context = new ScriptingContext (Interpreter);
+				context.InterruptionHandler = async;
+				context.CurrentFrame = frame;
+
+				if ((flags & EE.EvaluationFlags.NestedBreakStates) != 0)
+					context.ScriptingFlags |= ScriptingFlags.NestedBreakStates;
+
+				object data;
+				EE.EvaluationResult result = DoEvaluate (context, expression.Expression, out data);
+				callback (result, data);
+				async.WaitHandle.Set ();
+			});
+
+			return async;
+		}
+
+		EE.EvaluationResult DoEvaluate (ScriptingContext context, Expression expression, out object result)
+		{
+			Expression resolved = null;
+
+			try {
+				resolved = expression.Resolve (context);
+				if (resolved == null) {
+					result = String.Format ("Cannot resolve expression `{0}'", expression.Name);
+					return EE.EvaluationResult.InvalidExpression;
+				}
+
+				result = resolved.Evaluate (context);
+				return EE.EvaluationResult.Ok;
+			} catch (InvocationException ex) {
+				result = ex.Exception;
+				return EE.EvaluationResult.Exception;
+			} catch (ScriptingException ex) {
+				result = ex.Message;
+				return EE.EvaluationResult.InvalidExpression;
+			} catch (EvaluationTimeoutException) {
+				result = null;
+				return EE.EvaluationResult.Timeout;
+			} catch (Exception ex) {
+				result = String.Format ("Cannot resolve expression `{0}': {1}", expression.Name, ex);
+				return EE.EvaluationResult.InvalidExpression;
+			}
+		}
+
+		protected class MyExpression : EE.IExpression
+		{
+			public readonly ExpressionParser Parser;
+			public readonly Expression Expression;
+
+			public string Name {
+				get { return Expression.Name; }
+			}
+
+			public MyExpression (ExpressionParser parser, Expression expr)
+			{
+				this.Parser = parser;
+				this.Expression = expr;
+			}
+
+			public EE.AsyncResult EvaluateAsync (StackFrame frame, EE.EvaluationFlags flags,
+							     EE.EvaluationCallback callback)
+			{
+				return Parser.EvaluateAsync (frame, this, flags, callback);
+			}
+
+			public override string ToString ()
+			{
+				return Expression.Name;
+			}
+		}
+
+		protected class MyAsyncResult : EE.AsyncResult, IInterruptionHandler
+		{
+			public readonly MyExpression Expression;
+			public readonly ST.ManualResetEvent WaitHandle;
+			public readonly ST.ManualResetEvent AbortHandle;
+
+			public MyAsyncResult (MyExpression expr)
+			{
+				this.Expression = expr;
+				this.WaitHandle = new ST.ManualResetEvent (false);
+				this.AbortHandle = new ST.ManualResetEvent (false);
+			}
+
+			public override object AsyncState {
+				get { return Expression; }
+			}
+
+			public override ST.WaitHandle AsyncWaitHandle {
+				get { return WaitHandle; }
+			}
+
+			public override bool CompletedSynchronously {
+				get { return false; }
+			}
+
+			public override bool IsCompleted {
+				get { return WaitHandle.WaitOne (0); }
+			}
+
+			public override void Abort ()
+			{
+				AbortHandle.Set ();
+			}
+
+			ST.WaitHandle IInterruptionHandler.InterruptionEvent {
+				get { return AbortHandle; }
+			}
+
+			bool IInterruptionHandler.CheckInterruption ()
+			{
+				return AbortHandle.WaitOne (0);
+			}
+		}
 	}
 
 	public abstract class Expression
@@ -483,14 +606,13 @@ namespace Mono.Debugger.Frontend
 
 		protected override TargetObject DoEvaluateObject (ScriptingContext context)
 		{
-			StackFrame frame = context.CurrentFrame;
-			if ((frame.Language == null) ||
-			    !frame.Language.CanCreateInstance (val.GetType ()))
+			if ((context.CurrentLanguage == null) ||
+			    !context.CurrentLanguage.CanCreateInstance (val.GetType ()))
 				throw new ScriptingException (
 					"Cannot instantiate value '{0}' in the current frame's " +
 					"language", Name);
 
-			return frame.Language.CreateInstance (frame.Thread, val);
+			return context.CurrentLanguage.CreateInstance (context.CurrentThread, val);
 		}
 
 		public override TargetAddress EvaluateAddress (ScriptingContext context)
@@ -535,12 +657,11 @@ namespace Mono.Debugger.Frontend
 
 		protected override TargetObject DoEvaluateObject (ScriptingContext context)
 		{
-			StackFrame frame = context.CurrentFrame;
-			if ((frame.Language == null) ||
-			    !frame.Language.CanCreateInstance (typeof (string)))
+			if ((context.CurrentLanguage == null) ||
+			    (!context.CurrentLanguage.CanCreateInstance (typeof (string))))
 				throw new ScriptingException ("Cannot instantiate value '{0}' in the current frame's language", Name);
 
-			return frame.Language.CreateInstance (frame.Thread, val);
+			return context.CurrentLanguage.CreateInstance (context.CurrentThread, val);
 		}
 
 		public override string ToString ()
@@ -575,12 +696,11 @@ namespace Mono.Debugger.Frontend
 
 		protected override TargetObject DoEvaluateObject (ScriptingContext context)
 		{
-			StackFrame frame = context.CurrentFrame;
-			if ((frame.Language == null) ||
-			    !frame.Language.CanCreateInstance (typeof (bool)))
+			if ((context.CurrentLanguage == null) ||
+			    !context.CurrentLanguage.CanCreateInstance (typeof (bool)))
 				throw new ScriptingException ("Cannot instantiate value '{0}' in the current frame's language", Name);
 
-			return frame.Language.CreateInstance (frame.Thread, val);
+			return context.CurrentLanguage.CreateInstance (context.CurrentThread, val);
 		}
 
 		public override string ToString ()
@@ -935,7 +1055,7 @@ namespace Mono.Debugger.Frontend
 			if (member != null)
 				return member;
 
-			string[] namespaces = context.GetNamespaces (frame);
+			string[] namespaces = context.GetNamespaces ();
 			if (namespaces == null)
 				return null;
 
@@ -966,6 +1086,22 @@ namespace Mono.Debugger.Frontend
 				TargetAddress address = context.CurrentProcess.LookupSymbol (name);
 				if (!address.IsNull)
 					return new NumberExpression (address.Address);
+			} else if (context.ImplicitInstance != null) {
+				MemberExpression member = StructAccessExpression.FindMember (
+					context.CurrentThread, context.ImplicitInstance.Type,
+					context.ImplicitInstance, name, true, true);
+				if (member != null)
+					return member;
+
+				string[] namespaces = context.GetNamespaces () ?? new string [0];
+				foreach (string ns in namespaces) {
+					string full_name = MakeFQN (ns, name);
+					member = StructAccessExpression.FindMember (
+						context.CurrentThread, context.ImplicitInstance.Type,
+						context.ImplicitInstance, full_name, true, true);
+					if (member != null)
+						return member;
+				}
 			}
 
 			SourceLocation location = context.FindMethod (name);
@@ -1011,7 +1147,7 @@ namespace Mono.Debugger.Frontend
 			if (type != null)
 				return new TypeExpression (type);
 
-			string[] namespaces = context.GetNamespaces (context.CurrentFrame);
+			string[] namespaces = context.GetNamespaces ();
 			if (namespaces == null)
 				return null;
 
@@ -1221,16 +1357,14 @@ namespace Mono.Debugger.Frontend
 
 		protected override Expression DoResolveType (ScriptingContext context)
 		{
-			StackFrame frame = context.CurrentFrame;
-
 			TargetType the_type;
 
 			Expression ltype = left.TryResolveType (context);
 			if (ltype == null)
-				the_type = frame.Language.LookupType (Name);
+				the_type = context.CurrentLanguage.LookupType (Name);
 			else {
 				string nested = ltype.Name + "+" + name;
-				the_type = frame.Language.LookupType (nested);
+				the_type = context.CurrentLanguage.LookupType (nested);
 			}
 
 			if (the_type == null)
@@ -1524,6 +1658,7 @@ namespace Mono.Debugger.Frontend
 		private long GetValue (ScriptingContext context, Expression expr)
 		{
 			object val = expr.Evaluate (context);
+		again:
 			if (val is int)
 				return (long) (int) val;
 			else if (val is uint)
@@ -1535,8 +1670,14 @@ namespace Mono.Debugger.Frontend
 			else if (val is TargetPointerObject) {
 				TargetPointerObject pobj = (TargetPointerObject) val;
 				return pobj.GetAddress (context.CurrentThread).Address;
-			} else
-				throw new ScriptingException ("Cannot evaluate expression `{0}'", expr);
+			} else if (val is TargetFundamentalObject) {
+				TargetFundamentalObject fobj = (TargetFundamentalObject) val;
+				val = fobj.GetObject (context.CurrentThread);
+				if (!(val is TargetFundamentalObject))
+					goto again;
+			}
+
+			throw new ScriptingException ("Cannot evaluate expression `{0}'", expr.Name);
 		}
 
 		protected override object DoEvaluate (ScriptingContext context)
@@ -1547,6 +1688,8 @@ namespace Mono.Debugger.Frontend
 			try {
 				long retval = DoEvaluate (context, lvalue, rvalue);
 				return new NumberExpression (retval);
+			} catch (DivideByZeroException) {
+				throw new ScriptingException ("DivisionByZero");
 			} catch {
 				throw new ScriptingException ("Cannot evaluate expression `{0}'", Name);
 			}
@@ -1729,7 +1872,7 @@ namespace Mono.Debugger.Frontend
 			if (class_info != null)
 				return;
 
-			class_info = Type.GetClass (target);
+			class_info = Type.ForceClassInitialization (target);
 			if (class_info == null)
 				throw new ScriptingException ("Class `{0}' not initialized yet.",
 							      Type.Name);
@@ -1790,9 +1933,11 @@ namespace Mono.Debugger.Frontend
 		protected TargetObject GetProperty (ScriptingContext context,
 						    TargetPropertyInfo prop)
 		{
-			RuntimeInvokeResult result = context.Interpreter.RuntimeInvoke (
+			RuntimeInvokeFlags flags = context.GetRuntimeInvokeFlags ();
+
+			RuntimeInvokeResult result = context.RuntimeInvoke (
 				context.CurrentThread, prop.Getter, InstanceObject,
-				new TargetObject [0], true, false);
+				new TargetObject [0], flags);
 
 			if (result.ExceptionMessage != null)
 				throw new ScriptingException (
@@ -1922,14 +2067,12 @@ namespace Mono.Debugger.Frontend
 
 		protected override TargetObject DoEvaluateObject (ScriptingContext context)
 		{
-			StackFrame frame = context.CurrentFrame;
-
 			if (!Member.IsStatic && (InstanceObject == null))
 				throw new ScriptingException (
 					"Instance member `{0}' cannot be used in static context.", Name);
 
 			try {
-				return GetMember (context, frame.Thread, Member);
+				return GetMember (context, context.CurrentThread, Member);
 			} catch (TargetException ex) {
 				throw new ScriptingException ("Cannot access struct member `{0}': {1}",
 							      Name, ex.Message);
@@ -2012,9 +2155,11 @@ namespace Mono.Debugger.Frontend
 			if (prop.Setter == null)
 				throw new ScriptingException ("Property `{0}' has no setter.", Name);
 
-			RuntimeInvokeResult result = context.Interpreter.RuntimeInvoke (
+			RuntimeInvokeFlags flags = context.GetRuntimeInvokeFlags ();
+
+			RuntimeInvokeResult result = context.RuntimeInvoke (
 				context.CurrentThread, prop.Setter, InstanceObject,
-				new TargetObject [] { obj }, true, false);
+				new TargetObject [] { obj }, flags);
 
 			if (result.ExceptionMessage != null)
 				throw new ScriptingException (
@@ -2025,20 +2170,10 @@ namespace Mono.Debugger.Frontend
 		protected override bool DoAssign (ScriptingContext context, TargetObject obj)
 		{
 			if (Member is TargetFieldInfo) {
-				if (Member.Type != obj.Type)
-					throw new ScriptingException (
-						"Type mismatch: cannot assign expression of type " +
-						"`{0}' to field `{1}', which is of type `{2}'.",
-						obj.TypeName, Name, Member.Type.Name);
-
+				obj = Convert.ImplicitConversionRequired (context, obj, Member.Type);
 				SetField (context.CurrentThread, (TargetFieldInfo) Member, obj);
 			} else if (Member is TargetPropertyInfo) {
-				if (Member.Type != obj.Type)
-					throw new ScriptingException (
-						"Type mismatch: cannot assign expression of type " +
-						"`{0}' to property `{1}', which is of type `{2}'.",
-						obj.TypeName, Name, Member.Type.Name);
-
+				obj = Convert.ImplicitConversionRequired (context, obj, Member.Type);
 				SetProperty (context, (TargetPropertyInfo) Member, obj);
 			} else if (Member is TargetEventInfo)
 				throw new ScriptingException ("Can't set events directly.");
@@ -2696,6 +2831,11 @@ namespace Mono.Debugger.Frontend
 					return true;
 				break;
 
+			case FundamentalKind.Decimal:
+				if (tkind == FundamentalKind.Decimal)
+					return true;
+				break;
+
 			default:
 				break;
 			}
@@ -2736,6 +2876,8 @@ namespace Mono.Debugger.Frontend
 				return System.Convert.ToSingle (value);
 			case FundamentalKind.Double:
 				return System.Convert.ToDouble (value);
+			case FundamentalKind.Decimal:
+				return System.Convert.ToDecimal (value);
 			default:
 				return null;
 			}
@@ -2844,6 +2986,18 @@ namespace Mono.Debugger.Frontend
 					context, (TargetClassObject) obj,
 					(TargetClassType) type);
 
+			TargetNullableType ntype = type as TargetNullableType;
+			if (ntype != null) {
+				if (obj.Kind == TargetObjectKind.Null)
+					return obj;
+				else if (obj.Kind != TargetObjectKind.Nullable)
+					return ImplicitConversion (context, obj, ntype.ElementType);
+
+				TargetNullableType ntype2 = (TargetNullableType) obj.Type;
+				if (ImplicitConversionExists (context, ntype2.ElementType, ntype.ElementType))
+					return obj;
+			}
+
 			return null;
 		}
 
@@ -3155,17 +3309,19 @@ namespace Mono.Debugger.Frontend
 				Thread thread = context.CurrentThread;
 				RuntimeInvokeResult result;
 
-				result = context.Interpreter.RuntimeInvoke (
-					thread, method, instance, objs, true, debug);
+				RuntimeInvokeFlags flags = context.GetRuntimeInvokeFlags ();
+				if (debug)
+					flags |= RuntimeInvokeFlags.BreakOnEntry | RuntimeInvokeFlags.SendEventOnCompletion |
+						RuntimeInvokeFlags.NestedBreakStates;
+
+				result = context.RuntimeInvoke (thread, method, instance, objs, flags);
 
 				if (result == null)
 					throw new ScriptingException (
 						"Invocation of `{0}' aborted abnormally.", Name);
 
 				if (result.ExceptionMessage != null)
-					throw new ScriptingException (
-						"Invocation of `{0}' raised an exception: {1}",
-						Name, result.ExceptionMessage);
+					throw new InvocationException (Name, result.ExceptionMessage, result.ReturnObject);
 
 				return result.ReturnObject;
 			} catch (TargetException ex) {
@@ -3281,7 +3437,7 @@ namespace Mono.Debugger.Frontend
 			if (right is NullExpression) {
 				StackFrame frame = context.CurrentFrame;
 				TargetType ltype = left.EvaluateType (context);
-				obj = frame.Language.CreateNullObject (frame.Thread, ltype);
+				obj = context.CurrentLanguage.CreateNullObject (frame.Thread, ltype);
 			} else
 				obj = right.EvaluateObject (context);
 
@@ -3341,4 +3497,49 @@ namespace Mono.Debugger.Frontend
 			return sobj;
 		}
 	}
+
+	internal class TypeProxyExpression : Expression
+	{
+		string proxy_type;
+		TargetStructObject instance;
+
+		public TypeProxyExpression (string proxy_type, TargetStructObject instance)
+		{
+			this.proxy_type = proxy_type;
+			this.instance = instance;
+		}
+
+		public override string Name {
+			get { return String.Format ("[DebuggerTypeProxy(typeof({0}))]", proxy_type); }
+		}
+
+		TargetType LookupType (ScriptingContext context)
+		{
+			TargetType type = context.CurrentLanguage.LookupType (proxy_type);
+			if (type != null)
+				return type;
+
+			string[] namespaces = context.GetNamespaces () ?? new string [0];
+			foreach (string ns in namespaces) {
+				string full_name = SimpleNameExpression.MakeFQN (ns, proxy_type);
+				type = context.CurrentLanguage.LookupType (full_name);
+				if (type != null)
+					return type;
+			}
+
+			return null;
+		}
+
+		protected override Expression DoResolve (ScriptingContext context)
+		{
+			TargetType type = LookupType (context);
+			if (type == null)
+				return null;
+
+			Expression[] args = { new ArgumentExpression (instance) };
+			NewExpression expr = new NewExpression (new TypeExpression (type), args);
+
+			return expr.Resolve (context);
+		}
+	} 
 }
