@@ -6,18 +6,20 @@ using System.Reflection;
 using System.Collections;
 using System.Collections.Generic;
 using System.Globalization;
+using System.Runtime.Remoting;
 using System.Runtime.Serialization;
 using System.Runtime.InteropServices;
 using System.Runtime.Serialization.Formatters.Binary;
 
 using Mono.Debugger;
 using Mono.Debugger.Languages;
+using EE=Mono.Debugger.ExpressionEvaluator;
 
 using Mono.GetOptions;
 
 namespace Mono.Debugger.Frontend
 {
-	public class Interpreter : DebuggerMarshalByRefObject, IDisposable
+	public class Interpreter : DebuggerMarshalByRefObject, IInterruptionHandler, IDisposable
 	{
 		DebuggerConfiguration config;
 		DebuggerSession session;
@@ -71,8 +73,6 @@ namespace Mono.Debugger.Frontend
 			current_style = (StyleBase) styles ["cli"];
 		}
 
-		AppDomain debugger_domain;
-
 		public void Exit ()
 		{
 			try {
@@ -265,7 +265,7 @@ namespace Mono.Debugger.Frontend
 			}
 		}
 
-		protected void CheckLastEvent (Thread thread)
+		internal void CheckLastEvent (Thread thread)
 		{
 			if (thread == null)
 				return;
@@ -296,7 +296,7 @@ namespace Mono.Debugger.Frontend
 				current_process = main_process = debugger.Run (session);
 
 				current_thread = current_process.MainThread;
-				current_thread = WaitAll (current_thread);
+				current_thread = WaitAll (current_thread) ?? current_process.MainThread;
 
 				return current_process;
 			} catch (TargetException) {
@@ -383,12 +383,12 @@ namespace Mono.Debugger.Frontend
 
 				return current_process;
 			} catch (TargetException ex) {
-				Console.WriteLine ("FUCK: {0}", ex);
+				Console.WriteLine ("Got a TargetException during LoadSession: {0}", ex);
 				debugger.Dispose ();
 				debugger = null;
 				throw;
 			} catch (Exception ex) {
-				Console.WriteLine ("FUCK: {0}", ex);
+				Console.WriteLine ("Got an Exception during LoadSession: {0}", ex);
 				debugger.Dispose ();
 				debugger = null;
 				throw;
@@ -460,7 +460,8 @@ namespace Mono.Debugger.Frontend
 			again:
 				Process[] processes = HasTarget ? debugger.Processes : new Process [0];
 				foreach (Process process in processes) {
-					foreach (Thread t in process.GetThreads ()) {
+					Thread[] the_threads = process.GetThreads ();
+					foreach (Thread t in the_threads) {
 						if ((t == thread) || seen_threads.Contains (t))
 							continue;
 
@@ -493,6 +494,8 @@ namespace Mono.Debugger.Frontend
 					result.CompletedEvent.WaitOne ();
 					break;
 				} else if (ret == 1) {
+					seen_threads = new Hashtable ();
+					wait_list = new Hashtable ();
 					process_event.Reset ();
 					goto again;
 				}
@@ -545,32 +548,6 @@ namespace Mono.Debugger.Frontend
 			WaitHandle.WaitAny (handles);
 		}
 
-		public RuntimeInvokeResult RuntimeInvoke (Thread thread,
-							  TargetFunctionType function,
-							  TargetStructObject object_argument,
-							  TargetObject[] param_objects,
-							  bool is_virtual, bool debug)
-		{
-			RuntimeInvokeResult result = thread.RuntimeInvoke (
-				function, object_argument, param_objects, is_virtual, debug);
-
-			if (DebuggerConfiguration.BrokenThreading) {
-				Thread ret = WaitAll (thread, result, false);
-				if (ret != thread) {
-					result.Abort ();
-					return null;
-				}
-			} else if (debug) {
-				Wait (thread);
-
-				CheckLastEvent (thread);
-			} else {
-				Wait (result);
-			}
-
-			return result;
-		}
-
 		public int Interrupt ()
 		{
 			interrupt_event.Set ();
@@ -583,6 +560,15 @@ namespace Mono.Debugger.Frontend
 			interrupt_event.Reset ();
 		}
 
+		WaitHandle IInterruptionHandler.InterruptionEvent {
+			get { return interrupt_event; }
+		}
+
+		bool IInterruptionHandler.CheckInterruption ()
+		{
+			return interrupt_level > 0;
+		}
+
 		public void ShowBreakpoints ()
 		{
 			Event[] events = Session.Events;
@@ -822,10 +808,10 @@ namespace Mono.Debugger.Frontend
 		}
 
 		public int InsertExceptionCatchPoint (Thread target, ThreadGroup group,
-						      TargetType exception)
+						      TargetType exception, bool unhandled)
 		{
 			Event handle = target.Process.Session.InsertExceptionCatchPoint (
-				target, group, exception);
+				target, group, exception, unhandled);
 			return handle.Index;
 		}
 
@@ -992,6 +978,24 @@ namespace Mono.Debugger.Frontend
 				context.ShowDisplay (d);
 		}
 
+		internal CommandLineInterpreter CLI {
+			get; set;
+		}
+
+		protected virtual void OnEnterNestedBreakState (Thread thread)
+		{
+			if (CLI != null)
+				CLI.EnterNestedBreakState ();
+
+			CheckLastEvent (thread);
+		}
+
+		protected void OnLeaveNestedBreakState (Thread thread)
+		{
+			if (CLI != null)
+				CLI.LeaveNestedBreakState ();
+		}
+
 		//
 		// IDisposable
 		//
@@ -1116,6 +1120,14 @@ namespace Mono.Debugger.Frontend
 				this.interpreter = interpreter;
 
 				debugger.TargetEvent += target_event;
+				debugger.EnterNestedBreakStateEvent +=
+					delegate (Debugger unused, Thread thread) {
+						interpreter.OnEnterNestedBreakState (thread);
+					};
+				debugger.LeaveNestedBreakStateEvent +=
+					delegate (Debugger unused, Thread thread) {
+						interpreter.OnLeaveNestedBreakState (thread);
+					};
 			}
 
 			public void target_event (Thread thread, TargetEventArgs args)
