@@ -72,7 +72,10 @@ namespace Mono.Debugger.Languages
 
 		public abstract TargetObject CreateNullObject (Thread target, TargetType type);
 
+		public abstract TargetObjectObject CreateBoxedObject (Thread target, TargetObject value);
+
 		public abstract TargetPointerType CreatePointerType (TargetType type);
+
+		public abstract bool IsExceptionType (TargetClassType type);
 	}
 }
-
