@@ -12,13 +12,15 @@ namespace Mono.Debugger.Languages
 			this.Type = type;
 		}
 
-		public TargetObject Value {
-			get {
-				return Type.GetValue (Location);
-			}
+		public TargetObject GetValue (Thread thread)
+		{
+			return (TargetObject) thread.ThreadServant.DoTargetAccess (
+				delegate (TargetMemoryAccess target) {
+					return Type.Value.Type.GetObject (target, Location);
+			});
 		}
 
-		internal override long GetDynamicSize (Thread target, TargetBlob blob,
+		internal override long GetDynamicSize (TargetMemoryAccess target, TargetBlob blob,
 						       TargetLocation location,
 						       out TargetLocation dynamic_location)
 		{
