@@ -15,21 +15,7 @@ namespace Mono.Debugger.Languages
 			get;
 		}
 
-		public abstract bool HasSourceCode {
-			get;
-		}
-
-		public abstract SourceFile SourceFile {
-			get;
-		}
-
-		public abstract int StartRow {
-			get;
-		}
-
-		public abstract int EndRow {
-			get;
-		}
+		public abstract MethodSource GetSourceCode ();
 
 		public abstract bool IsStatic {
 			get;
@@ -63,7 +49,7 @@ namespace Mono.Debugger.Languages
 			get { return DeclaringType.Module; }
 		}
 
-		public abstract TargetClassType DeclaringType {
+		public abstract TargetStructType DeclaringType {
 			get;
 		}
 
@@ -75,5 +61,7 @@ namespace Mono.Debugger.Languages
 							 FunctionBreakpointHandle handle);
 
 		internal abstract void RemoveBreakpoint (Thread target);
+
+		public abstract TargetMethodSignature GetSignature (Thread target);
 	}
 }
