@@ -1,4 +1,5 @@
 using System;
+using System.Diagnostics;
 
 namespace Mono.Debugger.Languages
 {
@@ -30,6 +31,14 @@ namespace Mono.Debugger.Languages
 			this.Accessibility = accessibility;
 		}
 
+		public virtual DebuggerBrowsableState? DebuggerBrowsableState {
+			get { return null; }
+		}
+
+		public virtual DebuggerDisplayAttribute DebuggerDisplayAttribute {
+			get { return null; }
+		}
+
 		protected abstract string MyToString ();
 
 		public override string ToString ()
@@ -79,6 +88,10 @@ namespace Mono.Debugger.Languages
 			this.HasConstValue = has_const_value;
 		}
 
+		public abstract bool IsCompilerGenerated {
+			get;
+		}
+
 		public abstract object ConstValue {
 			get;
 		}
