@@ -12,24 +12,32 @@ namespace Mono.Debugger.Languages
 			this.Type = type;
 		}
 
-		public abstract TargetClassObject GetClassObject (Thread target);
+		public TargetClassObject GetClassObject (Thread thread)
+		{
+			return (TargetClassObject) thread.ThreadServant.DoTargetAccess (
+				delegate (TargetMemoryAccess target) {
+					return GetClassObject (target);
+			});
+		}
 
-		internal override long GetDynamicSize (Thread target, TargetBlob blob,
+		internal abstract TargetClassObject GetClassObject (TargetMemoryAccess target);
+
+		internal override long GetDynamicSize (TargetMemoryAccess target, TargetBlob blob,
 						       TargetLocation location,
 						       out TargetLocation dynamic_location)
 		{
 			throw new InvalidOperationException ();
 		}
 
-		public override TargetObject GetArrayElement (Thread target, int index)
+		internal override TargetObject GetArrayElement (TargetMemoryAccess target, int index)
 		{
 			throw new InvalidOperationException ();
 		}
 
-		public override string Print (Thread target)
+		internal override string Print (TargetMemoryAccess target)
 		{
 			if (HasAddress)
-				return String.Format ("{0} ({1})", Type.Name, Address);
+				return String.Format ("{0} ({1})", Type.Name, GetAddress (target));
 			else
 				return String.Format ("{0} ({1})", Type.Name, Location);
 		}
