@@ -1,3 +1,6 @@
+using System;
+using System.Diagnostics;
+
 namespace Mono.Debugger.Languages
 {
 	public abstract class TargetStructType : TargetType
@@ -6,6 +9,10 @@ namespace Mono.Debugger.Languages
 			: base (language, kind)
 		{ }
 
+		public abstract string BaseName {
+			get;
+		}
+
 		public abstract Module Module {
 			get;
 		}
@@ -14,6 +21,14 @@ namespace Mono.Debugger.Languages
 			get;
 		}
 
+		public virtual DebuggerDisplayAttribute DebuggerDisplayAttribute {
+			get { return null; }
+		}
+
+		public virtual DebuggerTypeProxyAttribute DebuggerTypeProxyAttribute {
+			get { return null; }
+		}
+
 		internal abstract TargetStructType GetParentType (TargetMemoryAccess target);
 
 		public TargetStructType GetParentType (Thread thread)
@@ -33,5 +48,10 @@ namespace Mono.Debugger.Languages
 					return GetClass (target);
 			});
 		}
+
+		public virtual TargetClass ForceClassInitialization (Thread thread)
+		{
+			return GetClass (thread);
+		}
 	}
 }
