@@ -11,9 +11,7 @@ namespace Mono.Debugger.Languages.Mono
 {
 	internal class MonoFunctionType : TargetFunctionType
 	{
-		MonoClassType klass;
-		SourceFile file;
-		int start_row, end_row;
+		IMonoStructType klass;
 		Cecil.MethodDefinition method_info;
 		TargetType return_type;
 		TargetType[] parameter_types;
@@ -22,8 +20,9 @@ namespace Mono.Debugger.Languages.Mono
 		int token;
 
 		int load_handler;
+		MonoMethodSignature signature;
 
-		internal MonoFunctionType (MonoClassType klass, Cecil.MethodDefinition mdef)
+		internal MonoFunctionType (IMonoStructType klass, Cecil.MethodDefinition mdef)
 			: base (klass.File.MonoLanguage)
 		{
 			this.klass = klass;
@@ -47,13 +46,9 @@ namespace Mono.Debugger.Languages.Mono
 					mdef.Parameters[i].ParameterType);
 		}
 
-		internal MonoFunctionType (MonoClassType klass, Cecil.MethodDefinition mdef,
-					   SourceFile file, int start_row, int end_row)
-			: this (klass, mdef)
+		public override MethodSource GetSourceCode ()
 		{
-			this.file = file;
-			this.start_row = start_row;
-			this.end_row = end_row;
+			return klass.File.GetMethodByToken (token);
 		}
 
 		internal static string GetMethodName (Cecil.MethodDefinition mdef)
@@ -78,7 +73,7 @@ namespace Mono.Debugger.Languages.Mono
 		}
 
 		public override string FullName {
-			get { return klass.Name + '.' + name; }
+			get { return klass.Type.Name + '.' + name; }
 		}
 
 		public override bool IsByRef {
@@ -109,12 +104,12 @@ namespace Mono.Debugger.Languages.Mono
 			get { return token; }
 		}
 
-		public override TargetClassType DeclaringType {
-			get { return klass; }
+		public override TargetStructType DeclaringType {
+			get { return klass.Type; }
 		}
 
-		internal MonoClassType MonoClass {
-			get { return klass; }
+		internal MonoSymbolFile SymbolFile {
+			get { return klass.File; }
 		}
 
 		internal Cecil.MethodDefinition MethodInfo {
@@ -133,34 +128,16 @@ namespace Mono.Debugger.Languages.Mono
 			get { return klass.File.TargetMemoryInfo.TargetAddressSize; }
 		}
 
-		public override bool HasSourceCode {
-			get { return file != null; }
-		}
-
-		public override SourceFile SourceFile {
+		public override bool ContainsGenericParameters {
 			get {
-				if (!HasSourceCode)
-					throw new InvalidOperationException ();
+				if (return_type.ContainsGenericParameters)
+					return true;
 
-				return file;
-			}
-		}
+				foreach (TargetType type in parameter_types)
+					if (type.ContainsGenericParameters)
+						return true;
 
-		public override int StartRow {
-			get {
-				if (!HasSourceCode)
-					throw new InvalidOperationException ();
-
-				return start_row;
-			}
-		}
-
-		public override int EndRow {
-			get {
-				if (!HasSourceCode)
-					throw new InvalidOperationException ();
-
-				return end_row;
+				return false;
 			}
 		}
 
@@ -189,5 +166,37 @@ namespace Mono.Debugger.Languages.Mono
 				load_handler = -1;
 			}
 		}
+
+		internal MonoClassInfo ResolveClass (TargetMemoryAccess target, bool fail)
+		{
+			return klass.ResolveClass (target, fail);
+		}
+
+		public override TargetMethodSignature GetSignature (Thread thread)
+		{
+			if (signature != null)
+				return signature;
+
+			if (!ContainsGenericParameters)
+				return new MonoMethodSignature (return_type, parameter_types);
+
+			TargetAddress addr = (TargetAddress) thread.ThreadServant.DoTargetAccess (
+				delegate (TargetMemoryAccess target)  {
+					MonoClassInfo class_info = ResolveClass (target, true);
+					return class_info.GetMethodAddress (target, token);
+			});
+
+			MonoLanguageBackend mono = klass.File.MonoLanguage;
+
+			TargetAddress sig = thread.CallMethod (
+				mono.MonoDebuggerInfo.GetMethodSignature, addr, 0);
+
+			signature = (MonoMethodSignature) thread.ThreadServant.DoTargetAccess (
+				delegate (TargetMemoryAccess target)  {
+					return mono.MonoRuntime.GetMethodSignature (mono, target, sig);
+			});
+
+			return signature;
+		}
 	}
 }
