@@ -39,6 +39,7 @@ namespace Mono.Debugger.Languages.Mono
 		public readonly MonoFundamentalType SingleType;
 		public readonly MonoFundamentalType DoubleType;
 		public readonly MonoFundamentalType CharType;
+		public readonly MonoFundamentalType DecimalType;
 		public readonly MonoStringType StringType;
 		public readonly MonoClassType ExceptionType;
 		public readonly MonoClassType DelegateType;
@@ -85,6 +86,11 @@ namespace Mono.Debugger.Languages.Mono
 			UIntType = MonoFundamentalType.Create (
 				corlib, memory, FundamentalKind.UIntPtr);
 
+			DecimalType = MonoFundamentalType.Create (
+				corlib, memory, FundamentalKind.Decimal);
+			Cecil.TypeDefinition decimal_type = corlib.ModuleDefinition.Types ["System.Decimal"];
+			corlib.AddType (decimal_type, DecimalType);
+
 			TargetAddress klass = corlib.MonoLanguage.MonoRuntime.GetArrayClass (memory);
 			Cecil.TypeDefinition array_type = corlib.ModuleDefinition.Types ["System.Array"];
 			ArrayType = mono.CreateCoreType (corlib, array_type, memory, klass);
@@ -575,18 +581,14 @@ namespace Mono.Debugger.Languages.Mono
 				TargetAddress klass = MonoRuntime.MonoArrayTypeGetClass (memory, data);
 				int rank = MonoRuntime.MonoArrayTypeGetRank (memory, data);
 
-				int numsizes = MonoRuntime.MonoArrayTypeGetNumSizes (memory, data);
-				int numlobounds = MonoRuntime.MonoArrayTypeGetNumLoBounds (memory, data);
-
-				if ((numsizes != 0) || (numlobounds != 0))
-					throw new InternalError ();
+				MonoRuntime.MonoArrayTypeGetBounds (memory, data);
 
 				TargetType etype = ReadMonoClass (memory, klass);
 				return new MonoArrayType (etype, rank);
 			}
 
 			case MonoTypeEnum.MONO_TYPE_GENERICINST:
-				return ReadGenericClass (memory, data);
+				return ReadGenericClass (memory, data, true);
 
 			case MonoTypeEnum.MONO_TYPE_VAR:
 			case MonoTypeEnum.MONO_TYPE_MVAR: {
@@ -602,8 +604,8 @@ namespace Mono.Debugger.Languages.Mono
 			}
 		}
 
-		public MonoGenericInstanceType ReadGenericClass (TargetMemoryAccess memory,
-								 TargetAddress address)
+		public TargetType ReadGenericClass (TargetMemoryAccess memory, TargetAddress address,
+						    bool handle_nullable)
 		{
 			MonoRuntime.GenericClassInfo info = MonoRuntime.GetGenericClass (memory, address);
 			if (info == null)
@@ -622,6 +624,9 @@ namespace Mono.Debugger.Languages.Mono
 			if (container == null)
 				return null;
 
+			if (handle_nullable && (container.Type.FullName == "System.Nullable`1"))
+				return new MonoNullableType (args [0]);
+
 			return new MonoGenericInstanceType (container, args, info.KlassPtr);
 		}
 
@@ -700,12 +705,13 @@ namespace Mono.Debugger.Languages.Mono
 				return null;
 
 			if (!assembly_by_name.Contains (symfile.Assembly.Name.FullName)) {
-				symfile_by_image_addr.Add (symfile.MonoImage, symfile);
 				assembly_hash.Add (symfile.Assembly, symfile);
 				assembly_by_name.Add (symfile.Assembly.Name.FullName, symfile);
-				symfile_by_index.Add (symfile.Index, symfile);
 			}
 
+			symfile_by_image_addr.Add (symfile.MonoImage, symfile);
+			symfile_by_index.Add (symfile.Index, symfile);
+
 			return symfile;
 		}
 
@@ -983,9 +989,13 @@ namespace Mono.Debugger.Languages.Mono
 			return file.ReadRangeEntry (target, reader, contents);
 		}
 
-		internal void RegisterMethodLoadHandler (int index, MethodLoadedHandler handler)
+		internal void RegisterMethodLoadHandler (TargetAccess target, TargetAddress info, int index,
+							 MethodLoadedHandler handler)
 		{
-			method_load_handlers.Add (index, handler);
+			if (!info.IsNull)
+				method_from_jit_info (target, info, handler);
+			else
+				method_load_handlers.Add (index, handler);
 		}
 
 		internal void RemoveMethodLoadHandler (Thread target, int index)
@@ -1101,6 +1111,8 @@ namespace Mono.Debugger.Languages.Mono
 				else if (type == typeof (UIntPtr))
 					return builtin_types.UIntType;
 				return null;
+			case TypeCode.Decimal:
+				return builtin_types.DecimalType;
 
 			default:
 				return null;
@@ -1288,6 +1300,71 @@ namespace Mono.Debugger.Languages.Mono
 			read_builtin_types (memory);
 		}
 
+		Dictionary<int,MonoRuntime.AppDomainInfo> appdomain_info = new Dictionary<int,MonoRuntime.AppDomainInfo> ();
+
+		void create_appdomain (TargetMemoryAccess memory, TargetAddress address)
+		{
+			int addr_size = memory.TargetMemoryInfo.TargetAddressSize;
+			TargetReader reader = new TargetReader (
+				memory.ReadMemory (address, 8 + 3 * addr_size));
+
+			int id = reader.BinaryReader.ReadInt32 ();
+			int shadow_path_len = reader.BinaryReader.ReadInt32 ();
+			TargetAddress shadow_path_addr = reader.ReadAddress ();
+
+			string shadow_path = null;
+			if (!shadow_path_addr.IsNull) {
+				byte[] buffer = memory.ReadBuffer (shadow_path_addr, shadow_path_len);
+				char[] cbuffer = new char [buffer.Length];
+				for (int i = 0; i < buffer.Length; i++)
+					cbuffer [i] = (char) buffer [i];
+				shadow_path = new String (cbuffer);
+			}
+
+			TargetAddress domain = reader.ReadAddress ();
+			TargetAddress setup = reader.ReadAddress ();
+
+			MonoRuntime.AppDomainInfo info = MonoRuntime.GetAppDomainInfo (this, memory, setup);
+			info.ShadowCopyPath = shadow_path;
+
+			appdomain_info.Add (id, info);
+		}
+
+		void unload_appdomain (int id)
+		{
+			appdomain_info.Remove (id);
+		}
+
+		bool is_shadow_copy_path (string path)
+		{
+			foreach (MonoRuntime.AppDomainInfo info in appdomain_info.Values) {
+				if (!info.ShadowCopyFiles || (info.ShadowCopyPath == null))
+					continue;
+
+				if (path.StartsWith (info.ShadowCopyPath))
+					return true;
+			}
+
+			return false;
+		}
+
+		internal string GetShadowCopyLocation (string path)
+		{
+			if (!is_shadow_copy_path (path))
+				return null;
+
+			string shadow_ini_path = Path.Combine (Path.GetDirectoryName (path), "__AssemblyInfo__.ini");
+			if (!File.Exists (shadow_ini_path))
+				return null;
+
+			try {
+				using (StreamReader ini_reader = File.OpenText (shadow_ini_path))
+					return ini_reader.ReadToEnd ();
+			} catch {
+				return null;
+			}
+		}
+
 		public bool Notification (SingleSteppingEngine engine, Inferior inferior,
 					  NotificationType type, TargetAddress data, long arg)
 		{
@@ -1310,13 +1387,13 @@ namespace Mono.Debugger.Languages.Mono
 					      "Module load: {0} {1}", data, symfile);
 				engine.Process.Debugger.OnModuleLoaded (symfile.Module);
 				if ((builtin_types != null) && (symfile != null)) {
-					if (engine.OnModuleLoaded ())
+					if (engine.OnModuleLoaded (symfile.Module))
 						return false;
 				}
 				break;
 			}
 
-			case NotificationType.UnloadModule:
+			case NotificationType.UnloadModule: {
 				Report.Debug (DebugFlags.JitSymtab,
 					      "Module unload: {0} {1}", data, arg);
 
@@ -1327,6 +1404,7 @@ namespace Mono.Debugger.Languages.Mono
 				engine.Process.Debugger.OnModuleUnLoaded (symfile.Module);
 				close_symfile (symfile);
 				break;
+			}
 
 			case NotificationType.JitBreakpoint:
 				JitBreakpoint (inferior, (int) arg, data);
@@ -1345,6 +1423,17 @@ namespace Mono.Debugger.Languages.Mono
 				engine.Process.BreakpointManager.DomainUnload (inferior, (int) arg);
 				break;
 
+			case NotificationType.ClassInitialized:
+				break;
+
+			case NotificationType.CreateAppDomain:
+				create_appdomain (inferior, data);
+				break;
+
+			case NotificationType.UnloadAppDomain:
+				unload_appdomain ((int) arg);
+				break;
+
 			default:
 				Console.WriteLine ("Received unknown notification {0:x} / {1} {2:x}",
 						   (int) type, data, arg);
