@@ -12,24 +12,24 @@ namespace Mono.Debugger.Languages.Mono
 			this.Type = type;
 		}
 
-		public override TargetClassObject GetClassObject (Thread target)
+		internal override TargetClassObject GetClassObject (TargetMemoryAccess target)
 		{
-			return (TargetClassObject) Type.ClassType.GetObject (Location);
+			return (TargetClassObject) Type.ClassType.GetObject (target, Location);
 		}
 
-		public override TargetType GetCurrentType (Thread target)
+		internal override TargetType GetCurrentType (TargetMemoryAccess target)
 		{
 			// location.Address resolves to the address of the MonoObject,
 			// dereferencing it once gives us the vtable, dereferencing it
 			// twice the class.
 			TargetAddress address;
-			address = target.TargetMemoryAccess.ReadAddress (Location.Address);
-			address = target.TargetMemoryAccess.ReadAddress (address);
+			address = target.ReadAddress (Location.GetAddress (target));
+			address = target.ReadAddress (address);
 
-			return Type.File.MonoLanguage.GetClass (target, address);
+			return Type.File.MonoLanguage.ReadMonoClass (target, address);
 		}
 
-		public override TargetObject GetDereferencedObject (Thread target)
+		internal override TargetObject GetDereferencedObject (TargetMemoryAccess target)
 		{
 			TargetType current_type = GetCurrentType (target);
 			if (current_type == null)
@@ -42,7 +42,7 @@ namespace Mono.Debugger.Languages.Mono
 
 			int offset = current_type.IsByRef ? 0 : type.Size;
 			TargetLocation new_location = Location.GetLocationAtOffset (offset);
-			TargetObject obj = current_type.GetObject (new_location);
+			TargetObject obj = current_type.GetObject (target, new_location);
 			return obj;
 		}
 	}
