@@ -216,6 +216,29 @@ namespace Mono.Debugger.Languages.Mono
 			return memory.ReadByte (atype + memory.TargetAddressSize + 2);
 		}
 
+		internal void MonoArrayTypeGetBounds (TargetMemoryAccess memory,
+						      TargetAddress data)
+		{
+			//
+			// FIXME: Only check whether the low bounds are all zero
+			//
+			int num_sizes = memory.ReadByte (data + memory.TargetAddressSize + 1);
+			if (num_sizes != 0)
+				throw new InternalError ();
+
+			int num_lobounds = memory.ReadByte (data + memory.TargetAddressSize + 2);
+			if (num_lobounds == 0)
+				return;
+
+			TargetAddress array = memory.ReadAddress (data + 3 * memory.TargetAddressSize);
+			TargetBinaryReader bounds = memory.ReadMemory (array, num_lobounds * 4).GetReader ();
+			for (int i = 0; i < num_lobounds; i++) {
+				int bound = bounds.ReadInt32 ();
+				if (bound != 0)
+					throw new InternalError ();
+			}
+		}
+
 		//
 		// Fundamental types
 		//
@@ -512,8 +535,14 @@ namespace Mono.Debugger.Languages.Mono
 			reader.ReadAddress ();
 			TargetAddress cached_class = reader.ReadAddress ();
 
+			int inst_id = memory.ReadInteger (class_inst);
 			int inst_data = memory.ReadInteger (class_inst + 4);
-			TargetAddress inst_argv = memory.ReadAddress (class_inst + 8);
+
+			TargetAddress inst_argv;
+			if (MonoDebuggerInfo.MajorVersion == 80)
+				inst_argv = memory.ReadAddress (class_inst + 8);
+			else
+				inst_argv = class_inst + 8;
 
 			int type_argc = inst_data & 0x3fffff;
 
@@ -586,5 +615,49 @@ namespace Mono.Debugger.Languages.Mono
 				this.Position = pos;
 			}
 		}
+
+		public AppDomainInfo GetAppDomainInfo (MonoLanguageBackend mono, TargetMemoryAccess memory,
+						       TargetAddress address)
+		{
+			int addr_size = memory.TargetMemoryInfo.TargetAddressSize;
+			TargetReader reader = new TargetReader (memory.ReadMemory (address, 12 * addr_size));
+
+			return new AppDomainInfo (mono, memory, reader);
+		}
+
+		public class AppDomainInfo
+		{
+			public readonly string ApplicationBase;
+			public readonly string ApplicationName;
+			public readonly string CachePath;
+			public readonly string ConfigFile;
+			public readonly string DynamicBase;
+			public readonly string ShadowCopyDirectories;
+			public readonly bool ShadowCopyFiles;
+
+			public string ShadowCopyPath;
+
+			public AppDomainInfo (MonoLanguageBackend mono, TargetMemoryAccess memory, TargetReader reader)
+			{
+				int addr_size = memory.TargetMemoryInfo.TargetAddressSize;
+
+				reader.Offset = 2 * addr_size;
+				ApplicationBase = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ());
+				ApplicationName = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ());
+				CachePath = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ());
+				ConfigFile = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ());
+				DynamicBase = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ());
+				reader.Offset += 3 * addr_size;
+				ShadowCopyDirectories = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ());
+				ShadowCopyFiles = MonoStringObject.ReadString (mono, memory, reader.ReadAddress ()) == "true";
+			}
+
+			public override string ToString ()
+			{
+				return String.Format ("AppDomainInfo ({0}:{1}:{2}:{3}:{4}:{5}:{6})",
+						      ApplicationBase, ApplicationName, CachePath, ConfigFile,
+						      DynamicBase, ShadowCopyDirectories, ShadowCopyFiles);
+			}
+		}
 	}
 }
