@@ -1,4 +1,5 @@
 using System;
+using System.Diagnostics;
 using System.Collections;
 using System.Collections.Generic;
 using System.Globalization;
@@ -24,7 +25,8 @@ namespace Mono.Debugger.Languages.Mono
 		{
 			Register	= 0,
 			RegOffset	= 0x10000000,
-			TwoRegisters	= 0x20000000
+			TwoRegisters	= 0x20000000,
+			Dead		= 0x30000000
 		}
 
 		const long AddressModeFlags = 0xf0000000;
@@ -220,6 +222,10 @@ namespace Mono.Debugger.Languages.Mono
 
 			TypeTable = MonoTypeTable.CreateTypeTable (this, memory, type_table_ptr);
 
+			string shadow_location = language.GetShadowCopyLocation (ImageFile);
+			if (shadow_location != null)
+				ImageFile = shadow_location;
+
 			try {
 				Assembly = Cecil.AssemblyFactory.GetAssembly (ImageFile);
 			} catch (Exception ex) {
@@ -298,6 +304,10 @@ namespace Mono.Debugger.Languages.Mono
 			get { return name; }
 		}
 
+		public override string CodeBase {
+			get { return ImageFile; }
+		}
+
 		public override Language Language {
 			get { return MonoLanguage; }
 		}
@@ -405,6 +415,7 @@ namespace Mono.Debugger.Languages.Mono
 
 			foreach (C.SourceFileEntry source in File.Sources) {
 				string file_name = source.FileName;
+				string orig_name = file_name;
 				if (need_conversion)
 					file_name = DebuggerConfiguration.WindowsToUnix (file_name);
 
@@ -533,10 +544,21 @@ namespace Mono.Debugger.Languages.Mono
 			List<MethodSource> methods = new List<MethodSource> ();
 
 			foreach (C.MethodEntry method in File.Methods) {
-				if (method.CompileUnit.SourceFile.Index != source.Index)
+				if (method.CompileUnit.SourceFile.Index == source.Index) {
+					methods.Add (GetMethodSource (file, method.Index));
 					continue;
+				}
 
-				methods.Add (GetMethodSource (file, method.Index));
+				bool found = false;
+				foreach (C.SourceFileEntry include in method.CompileUnit.IncludeFiles) {
+					if (include.Index == source.Index) {
+						found = true;
+						break;
+					}
+				}
+
+				if (found)
+					methods.Add (GetMethodSource (file, method.Index));
 			}
 
 			return methods.ToArray ();
@@ -573,6 +595,7 @@ namespace Mono.Debugger.Languages.Mono
 			case "System.Double":	return "double";
 			case "System.String":	return "string";
 			case "System.Object":	return "object";
+			case "System.Decimal":  return "decimal";
 			default:		return RemoveGenericArity (t.FullName);
 			}
 		}
@@ -774,6 +797,49 @@ namespace Mono.Debugger.Languages.Mono
 		internal override void OnModuleChanged ()
 		{ }
 
+		const string cgen_attr = "System.Runtime.CompilerServices.CompilerGeneratedAttribute";
+		const string debugger_display_attr = "System.Diagnostics.DebuggerDisplayAttribute";
+		const string type_proxy_attr = "System.Diagnostics.DebuggerTypeProxyAttribute";
+		const string browsable_attr = "System.Diagnostics.DebuggerBrowsableAttribute";
+
+		internal static void CheckCustomAttributes (Cecil.ICustomAttributeProvider type,
+							    out DebuggerBrowsableState? browsable_state,
+							    out DebuggerDisplayAttribute debugger_display,
+							    out DebuggerTypeProxyAttribute type_proxy,
+							    out bool is_compiler_generated)
+		{
+			browsable_state = null;
+			debugger_display = null;
+			type_proxy = null;
+			is_compiler_generated = false;
+
+			foreach (Cecil.CustomAttribute cattr in type.CustomAttributes) {
+				string cname = cattr.Constructor.DeclaringType.FullName;
+				if (cname == cgen_attr) {
+					is_compiler_generated = true;
+				} else if (cname == debugger_display_attr) {
+					string text = (string) cattr.ConstructorParameters [0];
+					debugger_display = new DebuggerDisplayAttribute (text);
+					foreach (DictionaryEntry prop in cattr.Properties) {
+						string key = (string) prop.Key;
+						if (key == "Name")
+							debugger_display.Name = (string) prop.Value;
+						else if (key == "Type")
+							debugger_display.Type = (string) prop.Value;
+						else {
+							debugger_display = null;
+							break;
+						}
+					}
+				} else if (cname == browsable_attr) {
+					browsable_state = (DebuggerBrowsableState) cattr.Blob [2];
+				} else if (cname == type_proxy_attr) {
+					string text = (string) cattr.ConstructorParameters [0];
+					type_proxy = new DebuggerTypeProxyAttribute (text);
+				}
+			}
+		}
+
 		protected override void DoDispose ()
 		{
 			if (File != null)
@@ -981,6 +1047,31 @@ namespace Mono.Debugger.Languages.Mono
 			public override Method NativeMethod {
 				get { throw new InvalidOperationException (); }
 			}
+
+			public override string[] GetNamespaces ()
+			{
+				int index = method.NamespaceID;
+
+				Hashtable namespaces = new Hashtable ();
+
+				C.CompileUnitEntry source = method.CompileUnit;
+				foreach (C.NamespaceEntry nse in source.Namespaces)
+					namespaces.Add (nse.Index, nse);
+
+				ArrayList list = new ArrayList ();
+
+				while ((index > 0) && namespaces.Contains (index)) {
+					C.NamespaceEntry ns = (C.NamespaceEntry) namespaces [index];
+					list.Add (ns.Name);
+					list.AddRange (ns.UsingClauses);
+
+					index = ns.Parent;
+				}
+
+				string[] retval = new string [list.Count];
+				list.CopyTo (retval, 0);
+				return retval;
+			}
 		}
 
 		protected class MonoCodeBlock : Block
@@ -1194,13 +1285,17 @@ namespace Mono.Debugger.Languages.Mono
 					} else
 						var = address.LocalVariableInfo [sv.Index];
 
-					TargetStructType type = mono.ReadStructType (memory, var.MonoType);
-					MonoVariable scope_var = new MonoVariable (
-						"$__" + sv.Scope, type, true, type.IsByRef, this, var);
+					try {
+						TargetStructType type = mono.ReadStructType (memory, var.MonoType);
+						MonoVariable scope_var = new MonoVariable (
+							"$__" + sv.Scope, type, true, type.IsByRef, this, var);
 
-					ScopeInfo info = new ScopeInfo (sv.Scope, scope_var, type);
-					scopes.Add (sv.Scope, info);
-					scope_list.Add (info);
+						ScopeInfo info = new ScopeInfo (sv.Scope, scope_var, type);
+						scopes.Add (sv.Scope, info);
+						scope_list.Add (info);
+					} catch (Exception ex) {
+						Report.Error ("Cannot read scope variable: {0}\n{1}", var, ex);
+					}
 				}
 
 				foreach (ScopeInfo scope in scope_list) {
@@ -1496,6 +1591,46 @@ namespace Mono.Debugger.Languages.Mono
 				}
 			}
 
+			public override TargetAddress Lookup (int line, int column)
+			{
+				if (column == -1)
+					return Lookup (line);
+
+				if ((Addresses == null) || (line < StartRow) || (line > EndRow))
+					return TargetAddress.Null;
+
+				bool found_a_range = false;
+
+				for (int i = 0; i < Addresses.Length; i++) {
+					LineEntry entry = (LineEntry) Addresses [i];
+
+					if (entry.SourceRange == null)
+						continue;
+
+					SourceRange range = entry.SourceRange.Value;
+					found_a_range = true;
+
+					if ((line < range.StartLine) || (line > range.EndLine))
+						continue;
+					if ((line == range.StartLine) && (column < range.StartColumn))
+						continue;
+					if ((line == range.EndLine) && (column > range.EndColumn))
+						continue;
+
+					return entry.Address;
+				}
+
+				//
+				// If the symbol file doesn't contain any source ranges for this
+				// method, default to traditional line-based lookup.
+				//
+
+				if (!found_a_range)
+					return Lookup (line);
+
+				return TargetAddress.Null;
+			}
+
 			public override TargetAddress Lookup (int line)
 			{
 				if ((Addresses == null) || (line < StartRow) || (line > EndRow))
@@ -1545,7 +1680,7 @@ namespace Mono.Debugger.Languages.Mono
 						       (int) (Addresses [0].Address - address));
 			}
 
-			SourceAddress create_address (LineEntry entry, int offset, int range)
+			SourceAddress create_address (LineEntry entry, int line_offset, int line_range)
 			{
 				SourceFile file = null;
 				SourceBuffer buffer = null;
@@ -1559,7 +1694,7 @@ namespace Mono.Debugger.Languages.Mono
 						buffer = Method.MethodSource.SourceBuffer;
 				}
 
-				return new SourceAddress (file, buffer, entry.Line, offset, range);
+				return new SourceAddress (file, buffer, entry.Line, line_offset, line_range, entry.SourceRange);
 			}
 
 			public override void DumpLineNumbers ()
@@ -1633,7 +1768,12 @@ namespace Mono.Debugger.Languages.Mono
 						int file = lne.File != entry.CompileUnit.SourceFile.Index ? lne.File : 0;
 						bool hidden = lne.IsHidden;
 
-						lines.Add (new LineEntry (address, file, lne.Row, hidden));
+						SourceRange? range = null;
+						if (lne.SourceRange != null)
+							range = new SourceRange (lne.SourceRange.StartLine, lne.SourceRange.EndLine,
+										 lne.SourceRange.StartColumn, lne.SourceRange.EndColumn);
+
+						lines.Add (new LineEntry (address, file, lne.Row, hidden, range));
 						last_line = lne.Row;
 					}
 
@@ -1658,10 +1798,10 @@ namespace Mono.Debugger.Languages.Mono
 				int start_row = 0, end_row = 0;
 				if (lines.Count > 0) {
 					start_row = lines [0].Line;
-					end_row = lines [0].Line;
+					end_row = lines [lines.Count - 1].Line;
 
 					foreach (LineEntry line in lines) {
-						if (line.IsHidden || (line.File != 0))
+						if (line.IsHidden)
 							continue;
 
 						if (line.Line < start_row)
@@ -1893,6 +2033,11 @@ namespace Mono.Debugger.Languages.Mono
 			public override Method NativeMethod {
 				get { return wrapper; }
 			}
+
+			public override string[] GetNamespaces ()
+			{
+				return null;
+			}
 		}
 
 		protected class WrapperMethod : Method
