@@ -1,4 +1,5 @@
 using System;
+using System.Text;
 using Mono.Debugger.Backend;
 
 namespace Mono.Debugger.Languages.Native
@@ -18,6 +19,10 @@ namespace Mono.Debugger.Languages.Native
 			this.parameter_types = parameter_types;
 		}
 
+		internal NativeFunctionType (Language language)
+			: base (language)
+		{ }
+
 		public override bool HasClassType {
 			get { return false; }
 		}
@@ -34,20 +39,9 @@ namespace Mono.Debugger.Languages.Native
 			get { return name; }
 		}
 
-		public override bool HasSourceCode {
-			get { return false; }
-		}
-
-		public override SourceFile SourceFile {
-			get { throw new InvalidOperationException (); }
-		}
-
-		public override int StartRow {
-			get { throw new InvalidOperationException (); }
-		}
-
-		public override int EndRow {
-			get { throw new InvalidOperationException (); }
+		public override MethodSource GetSourceCode ()
+		{
+			return null;
 		}
 
 		public override int Size {
@@ -72,6 +66,10 @@ namespace Mono.Debugger.Languages.Native
 			}
 		}
 
+		public override bool ContainsGenericParameters {
+			get { return false; }
+		}
+
 		public override bool HasReturnValue {
 			get {
 				return return_type != Language.VoidType;
@@ -90,21 +88,57 @@ namespace Mono.Debugger.Languages.Native
 			}
 		}
 
+		internal void SetPrototype (TargetType ret_type, TargetType[] param_types)
+		{
+			this.return_type = ret_type;
+			this.parameter_types = param_types;
+
+			StringBuilder sb = new StringBuilder ();
+			sb.Append (ret_type.Name);
+			sb.Append (" (*) (");
+
+			for (int i = 0; i < param_types.Length; i++) {
+				if (i > 0)
+					sb.Append (",");
+				sb.Append (param_types [i].Name);
+			}
+
+			sb.Append (")");
+			name = sb.ToString ();
+		}
+
+		internal string GetPointerName ()
+		{
+			StringBuilder sb = new StringBuilder ();
+			sb.Append (return_type.Name);
+			sb.Append (" (**) (");
+
+			for (int i = 0; i < parameter_types.Length; i++) {
+				if (i > 0)
+					sb.Append (",");
+				sb.Append (parameter_types [i].Name);
+			}
+
+			sb.Append (")");
+			return sb.ToString ();
+		}
+
 		public override object MethodHandle {
 			get {
 				return null;
 			}
 		}
 
-		public override TargetClassType DeclaringType {
+		public override TargetStructType DeclaringType {
 			get {
 				return null;
 			}
 		}
 
-	        protected override TargetObject DoGetObject (TargetMemoryAccess target, TargetLocation location)
+	        protected override TargetObject DoGetObject (TargetMemoryAccess target,
+							     TargetLocation location)
 		{
-			throw new NotSupportedException ();
+			throw new InvalidOperationException ();
 		}
 
 		public override bool IsManaged {
@@ -121,5 +155,10 @@ namespace Mono.Debugger.Languages.Native
 		{
 			throw new InvalidOperationException ();
 		}
+
+		public override TargetMethodSignature GetSignature (Thread target)
+		{
+			throw new InvalidOperationException ();
+		}
 	}
 }
