@@ -24,7 +24,9 @@ namespace Mono.Debugger.Tests
 		ProcessCreated,
 		ProcessExecd,
 		ProcessExited,
-		TargetExited
+		TargetExited,
+		EnterNestedBreakState,
+		CommandDone
 	}
 
 	public class DebuggerEvent {
@@ -90,6 +92,8 @@ namespace Mono.Debugger.Tests
 
 			Style = style_nunit = new StyleNUnit (this);
 			style_nunit.TargetEventEvent += delegate (Thread thread, TargetEventArgs args) {
+				if (IgnoreThreadCreation && (args.Type == TargetEventType.TargetExited))
+					return;
 				AddEvent (new DebuggerEvent (DebuggerEventType.TargetEvent, thread, args));
 			};
 
@@ -106,6 +110,10 @@ namespace Mono.Debugger.Tests
 			get { return queue.Count > 0; }
 		}
 
+		public bool IgnoreThreadCreation {
+			get; set;
+		}
+
 		public DebuggerEvent Wait ()
 		{
 			for (int i = 0; i < 5; i++) {
@@ -122,7 +130,7 @@ namespace Mono.Debugger.Tests
 			return null;
 		}
 
-		protected void AddEvent (DebuggerEvent e)
+		internal void AddEvent (DebuggerEvent e)
 		{
 			lock (queue.SyncRoot) {
 				queue.Enqueue (e);
@@ -133,13 +141,15 @@ namespace Mono.Debugger.Tests
 		protected override void OnThreadCreated (Thread thread)
 		{
 			base.OnThreadCreated (thread);
-			AddEvent (new DebuggerEvent (DebuggerEventType.ThreadCreated, thread));
+			if (!IgnoreThreadCreation)
+				AddEvent (new DebuggerEvent (DebuggerEventType.ThreadCreated, thread));
 		}
 
 		protected override void OnThreadExited (Thread thread)
 		{
 			base.OnThreadExited (thread);
-			AddEvent (new DebuggerEvent (DebuggerEventType.ThreadExited, thread));
+			if (!IgnoreThreadCreation)
+				AddEvent (new DebuggerEvent (DebuggerEventType.ThreadExited, thread));
 		}
 
 		protected override void OnMainProcessCreated (Process process)
@@ -180,6 +190,12 @@ namespace Mono.Debugger.Tests
 				inferior_stdout.Add (line);
 		}
 
+		protected override void OnEnterNestedBreakState (Thread thread)
+		{
+			base.OnEnterNestedBreakState (thread);
+			AddEvent (new DebuggerEvent (DebuggerEventType.EnterNestedBreakState, thread));
+		}
+
 		public override void Print (string message)
 		{
 			if (!IsScript)
@@ -231,6 +247,8 @@ namespace Mono.Debugger.Tests
 			FileName = Path.GetFullPath (srcdir + src_file);
 
 			config = new DebuggerConfiguration ();
+			config.RedirectOutput = true;
+
 			options = CreateOptions (ExeFileName, args);
 
 			inferior_stdout = new LineReader ();
@@ -262,6 +280,29 @@ namespace Mono.Debugger.Tests
 			}
 		}
 
+		protected void AddSourceFile (string filename)
+		{
+			string pathname = Path.Combine (SourceDirectory, filename);
+			using (StreamReader reader = new StreamReader (pathname)) {
+				string text;
+				int line = 0;
+				while ((text = reader.ReadLine ()) != null) {
+					line++;
+
+					Match match = Regex.Match (text, mdb_line_re);
+					if (!match.Success)
+						continue;
+
+					string name = match.Groups [2].Value;
+					lines.Add (name, line);
+					if (match.Groups [1].Value == "BREAKPOINT") {
+						string location = String.Format ("{0}:{1}", pathname, line);
+						automatic_breakpoints [name] = AssertBreakpoint (location);
+					}
+				}
+			}
+		}
+
 		protected void AssertHitBreakpoint (Thread thread, string name, string function)
 		{
 			AssertHitBreakpoint (thread, automatic_breakpoints [name],
@@ -403,7 +444,7 @@ namespace Mono.Debugger.Tests
 				Assert.Fail ("No such command: `{0}'", text);
 
 			try {
-				return command.Execute (engine);
+				return command.Execute (engine.Interpreter);
 			} catch (ScriptingException ex) {
 				Assert.Fail ("Failed to execute command `{0}': {1}.",
 					     text, ex.Message);
@@ -429,7 +470,7 @@ namespace Mono.Debugger.Tests
 
 			string exception = "";
 			try {
-				command.Execute (engine);
+				command.Execute (engine.Interpreter);
 				Assert.Fail ("Execution of command `{0}' failed to throw " +
 					     "exception `{1}'.", command, exp_exception);
 			} catch (ScriptingException ex) {
@@ -446,6 +487,24 @@ namespace Mono.Debugger.Tests
 					     "but expected `{2}'.", command, exception, exp_exception);
 		}
 
+		public void AssertExecuteInBackground (string text)
+		{
+			parser.Reset ();
+			parser.Append (text);
+			Command command = parser.GetCommand ();
+			if (command == null)
+				Assert.Fail ("No such command: `{0}'", text);
+
+			ST.ThreadPool.QueueUserWorkItem (delegate {
+				try {
+					object result = command.Execute (engine.Interpreter);
+					Interpreter.AddEvent (new DebuggerEvent (DebuggerEventType.CommandDone, result));
+				} catch (Exception ex) {
+					Interpreter.AddEvent (new DebuggerEvent (DebuggerEventType.CommandDone, ex));
+				}
+			});
+		}
+
 		public void AssertFrame (Thread thread, string line, string function)
 		{
 			try {
@@ -499,6 +558,19 @@ namespace Mono.Debugger.Tests
 			Assert.AreEqual ("<method called from mdb>", frame.Name.ToString (),
 					 "Got frame `{0}', but expected a method " +
 					 "called from mdb.", frame);
+			Assert.AreEqual (FrameType.Callback, frame.Type);
+		}
+
+		public void AssertRuntimeInvokeFrame (StackFrame frame, int level, string func)
+		{
+			Assert.AreEqual (level, frame.Level,
+					 "Stack frame is from level {0}, but expected {1}.",
+					 level, frame.Level);
+			Assert.AreEqual (FrameType.RuntimeInvoke, frame.Type);
+			string exp = String.Format ("<Invocation of: {0}>", func);
+			Assert.AreEqual (exp, frame.Name.ToString (),
+					 "Got frame `{0}', but expected a method " +
+					 "called from mdb.", frame);
 		}
 
 		public void AssertTargetOutput (string line)
@@ -533,6 +605,53 @@ namespace Mono.Debugger.Tests
 				AssertFrame (exp_thread, exp_func, exp_line);
 		}
 
+		public void AssertRuntimeInvokeDone (Thread exp_thread, string exp_func, int exp_line)
+		{
+			TargetEventArgs args = AssertTargetEvent (
+				exp_thread, TargetEventType.RuntimeInvokeDone);
+
+			if (exp_func != null)
+				AssertFrame (exp_thread, exp_func, exp_line);
+		}
+
+		public void AssertNestedBreakState (Thread exp_thread, string exp_func, int exp_line)
+		{
+			bool seen_stopped = false, seen_enter = false;
+
+			while (!seen_stopped || !seen_enter) {
+				DebuggerEvent e = Interpreter.Wait ();
+				if (e == null)
+					Assert.Fail ("Time-out while waiting for nested break state.");
+
+				if (e.Type == DebuggerEventType.EnterNestedBreakState) {
+					if (e.Data != exp_thread)
+						Assert.Fail ("Received event {0} while waiting for nested break state.", e);
+
+						seen_enter = true;
+				} else if (e.Type == DebuggerEventType.TargetEvent) {
+					if (e.Data != exp_thread)
+						Assert.Fail ("Received event {0} while waiting for nested break state.", e);
+
+					TargetEventArgs args = (TargetEventArgs) e.Data2;
+					if ((args.Type == TargetEventType.TargetStopped) && ((int) args.Data  != 0))
+						Assert.Fail ("Received event {0} while waiting for {1} to stop.",
+							     args, exp_thread);
+					else if ((args.Type != TargetEventType.TargetStopped) &&
+						 (args.Type != TargetEventType.TargetHitBreakpoint) &&
+						 (args.Type != TargetEventType.Exception) &&
+						 (args.Type != TargetEventType.UnhandledException))
+						Assert.Fail ("Received event {0} while waiting for {1} to stop.",
+							     args, exp_thread);
+
+					seen_stopped = true;
+				} else
+					Assert.Fail ("Received event {0} while waiting for nested break state.", e);
+			}
+
+			if (exp_func != null)
+				AssertFrame (exp_thread, exp_func, exp_line);
+		}
+
 		public void AssertSegfault (Thread exp_thread, string exp_func, int exp_line)
 		{
 			TargetEventArgs args = AssertTargetEvent (
@@ -566,9 +685,15 @@ namespace Mono.Debugger.Tests
 			AssertFrame (exp_thread, exp_func, exp_line);
 		}
 
+		public void AssertCaughtUnhandledException (Thread exp_thread, string exp_func, int exp_line)
+		{
+			AssertTargetEvent (exp_thread, TargetEventType.UnhandledException);
+			AssertFrame (exp_thread, exp_func, exp_line);
+		}
+
 		public int AssertBreakpoint (int location)
 		{
-			return AssertBreakpoint (location.ToString ());
+			return AssertBreakpoint (String.Format ("{0}:{1}", FileName, location));
 		}
 
 		public int AssertBreakpoint (string location)
@@ -600,6 +725,17 @@ namespace Mono.Debugger.Tests
 			return (int) result;
 		}
 
+		public int AssertUnhandledCatchpoint (string location)
+		{
+			object result = AssertExecute ("catch -unhandled " + location);
+			if (result == null) {
+				Assert.Fail ("Failed to insert catchpoint.");
+				return -1;
+			}
+
+			return (int) result;
+		}
+
 		public Process AssertMainProcessCreated ()
 		{
 			DebuggerEvent e = AssertEvent (DebuggerEventType.MainProcessCreated);
@@ -673,9 +809,7 @@ namespace Mono.Debugger.Tests
 		object EvaluateExpression (ScriptingContext context, string expression)
 		{
 			try {
-				Expression expr = interpreter.ExpressionParser.Parse (expression);
-				if (expr == null)
-					Assert.Fail ("Cannot parse expression `{0}'.", expression);
+				Expression expr = context.ParseExpression (expression);
 
 				expr = expr.Resolve (context);
 				if (expr == null)
@@ -700,9 +834,7 @@ namespace Mono.Debugger.Tests
 		TargetType EvaluateExpressionType (ScriptingContext context, string expression)
 		{
 			try {
-				Expression expr = interpreter.ExpressionParser.Parse (expression);
-				if (expr == null)
-					Assert.Fail ("Cannot parse expression `{0}'.", expression);
+				Expression expr = context.ParseExpression (expression);
 
 				Expression resolved = expr.TryResolveType (context);
 				if (resolved != null)
@@ -840,6 +972,23 @@ namespace Mono.Debugger.Tests
 					     expression, text, exp_result);
 		}
 
+		public object EvaluateExpression (Thread thread, string expression)
+		{
+			try {
+				ScriptingContext context = GetContext (thread);
+				object obj = EvaluateExpression (context, expression);
+				if (obj == null)
+					Assert.Fail ("Failed to evaluate expression `{0}'", expression);
+				return obj;
+			} catch (AssertionException) {
+				throw;
+			} catch (Exception ex) {
+				Assert.Fail ("Failed to print expression `{0}': {1}",
+					     expression, ex);
+				return null;
+			}
+		}
+
 		public DebuggerEvent AssertEvent ()
 		{
 			DebuggerEvent e = Interpreter.Wait ();
@@ -908,18 +1057,20 @@ namespace Mono.Debugger.Tests
 
 		public void AssertTargetExited (Process process)
 		{
-			bool target_event = false;
 			bool process_exited = false;
-			bool thread_exited = false;
+			bool target_exited = false;
+			bool target_event = Interpreter.IgnoreThreadCreation;
+			bool thread_exited = Interpreter.IgnoreThreadCreation;
 
 			while (true) {
 				DebuggerEvent e = Interpreter.Wait ();
 				if (e == null)
 					Assert.Fail ("Time-out while waiting for target to exit.");
 
-				if (e.Type == DebuggerEventType.TargetExited)
+				if (e.Type == DebuggerEventType.TargetExited) {
+					target_exited = true;
 					break;
-				if (e.Type == DebuggerEventType.ThreadExited) {
+				} else if (e.Type == DebuggerEventType.ThreadExited) {
 					if (e.Data == process.MainThread)
 						thread_exited = true;
 					continue;
@@ -949,6 +1100,9 @@ namespace Mono.Debugger.Tests
 			if (!thread_exited)
 				Assert.Fail ("Did not receive `ThreadExitedEvent' while waiting for " +
 					     "target to exit.");
+			if (!target_exited)
+				Assert.Fail ("Did not receive `TargetExitedEvent' while waiting for " +
+					     "target to exit.");
 
 			AssertNoTargetOutput ();
 		}
